'use strict';

var crypto = require('crypto');

const webcrypto$2 = getCryptoModule$2();
function getCryptoModule$2() {
    let webcrypto = null;
    if (exists$2(crypto.webcrypto)) {
        webcrypto = crypto.webcrypto;
    }
    else if (exists$2(globalThis.crypto)) {
        webcrypto = globalThis.crypto;
    }
    else if (exists$2(window.crypto)) {
        webcrypto = window.crypto;
    }
    if (!exists$2(webcrypto)) {
        console.log('Search for webcrypto library failed!');
        console.log('crypto:', typeof crypto.webcrypto);
        console.log('globalThis.crypto:', typeof globalThis?.crypto);
        console.log('window.crypto:', typeof window?.crypto);
        throw new Error('Unable to find webcrypto library. If you are running in a NodeJs environment, try using version 19 or newer. If you are running in a browser environment, make sure that you are running in a secure context.');
    }
    if (!exists$2(webcrypto?.subtle)) {
        throw new Error('Webcrypto library is partially disabled! Make sure you are running within a secure environment. If you are getting this error in the browser, make sure https is enabled and/or you are testing locally using 127.0.0.1.');
    }
    return webcrypto;
}
function exists$2(x) {
    try {
        return (typeof x !== 'undefined' && x !== null);
    }
    catch {
        return false;
    }
}

/*! noble-secp256k1 - MIT License (c) 2019 Paul Miller (paulmillr.com) */
const _0n = BigInt(0);
const _1n = BigInt(1);
const _2n = BigInt(2);
const _3n = BigInt(3);
const _8n = BigInt(8);
const CURVE = Object.freeze({
    a: _0n,
    b: BigInt(7),
    P: BigInt('0xfffffffffffffffffffffffffffffffffffffffffffffffffffffffefffffc2f'),
    n: BigInt('0xfffffffffffffffffffffffffffffffebaaedce6af48a03bbfd25e8cd0364141'),
    h: _1n,
    Gx: BigInt('55066263022277343669578718895168534326250603453777594175500187360389116729240'),
    Gy: BigInt('32670510020758816978083085130507043184471273380659243275938904335757337482424'),
    beta: BigInt('0x7ae96a2b657c07106e64479eac3434e99cf0497512f58995c1396c28719501ee')
});
const divNearest = (a, b) => (a + b / _2n) / b;
const endo = {
    beta: BigInt('0x7ae96a2b657c07106e64479eac3434e99cf0497512f58995c1396c28719501ee'),
    splitScalar(k) {
        const { n } = CURVE;
        const a1 = BigInt('0x3086d221a7d46bcde86c90e49284eb15');
        const b1 = -_1n * BigInt('0xe4437ed6010e88286f547fa90abfe4c3');
        const a2 = BigInt('0x114ca50f7a8e2f3f657c1108d9d44cfd8');
        const b2 = a1;
        const POW_2_128 = BigInt('0x100000000000000000000000000000000');
        const c1 = divNearest(b2 * k, n);
        const c2 = divNearest(-b1 * k, n);
        let k1 = mod(k - c1 * a1 - c2 * a2, n);
        let k2 = mod(-c1 * b1 - c2 * b2, n);
        const k1neg = k1 > POW_2_128;
        const k2neg = k2 > POW_2_128;
        if (k1neg)
            k1 = n - k1;
        if (k2neg)
            k2 = n - k2;
        if (k1 > POW_2_128 || k2 > POW_2_128) {
            throw new Error('splitScalarEndo: Endomorphism failed, k=' + String(k));
        }
        return { k1neg, k1, k2neg, k2 };
    }
};
const fieldLen = 32;
const groupLen = 32;
const hashLen = 32;
const compressedLen = fieldLen + 1;
const uncompressedLen = 2 * fieldLen + 1;
function weierstrass(x) {
    const { a, b } = CURVE;
    const x2 = mod(x * x);
    const x3 = mod(x2 * x);
    return mod(x3 + a * x + b);
}
const USE_ENDOMORPHISM = CURVE.a === _0n;
class ShaError extends Error {
    constructor(message) {
        super(message);
    }
}
function assertJacPoint(other) {
    if (!(other instanceof JacobianPoint))
        throw new TypeError('JacobianPoint expected');
}
class JacobianPoint {
    constructor(x, y, z) {
        this.x = x;
        this.y = y;
        this.z = z;
    }
    static get BASE() {
        return new JacobianPoint(CURVE.Gx, CURVE.Gy, _1n);
    }
    static get ZERO() {
        return new JacobianPoint(_0n, _1n, _0n);
    }
    static fromAffine(p) {
        if (!(p instanceof Point$1)) {
            throw new TypeError('JacobianPoint#fromAffine: expected Point');
        }
        if (p.equals(Point$1.ZERO))
            return JacobianPoint.ZERO;
        return new JacobianPoint(p.x, p.y, _1n);
    }
    static toAffineBatch(points) {
        const toInv = invertBatch(points.map((p) => p.z));
        return points.map((p, i) => p.toAffine(toInv[i]));
    }
    static normalizeZ(points) {
        return JacobianPoint.toAffineBatch(points).map(JacobianPoint.fromAffine);
    }
    equals(other) {
        assertJacPoint(other);
        const { x: X1, y: Y1, z: Z1 } = this;
        const { x: X2, y: Y2, z: Z2 } = other;
        const Z1Z1 = mod(Z1 * Z1);
        const Z2Z2 = mod(Z2 * Z2);
        const U1 = mod(X1 * Z2Z2);
        const U2 = mod(X2 * Z1Z1);
        const S1 = mod(mod(Y1 * Z2) * Z2Z2);
        const S2 = mod(mod(Y2 * Z1) * Z1Z1);
        return U1 === U2 && S1 === S2;
    }
    negate() {
        return new JacobianPoint(this.x, mod(-this.y), this.z);
    }
    double() {
        const { x: X1, y: Y1, z: Z1 } = this;
        const A = mod(X1 * X1);
        const B = mod(Y1 * Y1);
        const C = mod(B * B);
        const x1b = X1 + B;
        const D = mod(_2n * (mod(x1b * x1b) - A - C));
        const E = mod(_3n * A);
        const F = mod(E * E);
        const X3 = mod(F - _2n * D);
        const Y3 = mod(E * (D - X3) - _8n * C);
        const Z3 = mod(_2n * Y1 * Z1);
        return new JacobianPoint(X3, Y3, Z3);
    }
    add(other) {
        assertJacPoint(other);
        const { x: X1, y: Y1, z: Z1 } = this;
        const { x: X2, y: Y2, z: Z2 } = other;
        if (X2 === _0n || Y2 === _0n)
            return this;
        if (X1 === _0n || Y1 === _0n)
            return other;
        const Z1Z1 = mod(Z1 * Z1);
        const Z2Z2 = mod(Z2 * Z2);
        const U1 = mod(X1 * Z2Z2);
        const U2 = mod(X2 * Z1Z1);
        const S1 = mod(mod(Y1 * Z2) * Z2Z2);
        const S2 = mod(mod(Y2 * Z1) * Z1Z1);
        const H = mod(U2 - U1);
        const r = mod(S2 - S1);
        if (H === _0n) {
            if (r === _0n) {
                return this.double();
            }
            else {
                return JacobianPoint.ZERO;
            }
        }
        const HH = mod(H * H);
        const HHH = mod(H * HH);
        const V = mod(U1 * HH);
        const X3 = mod(r * r - HHH - _2n * V);
        const Y3 = mod(r * (V - X3) - S1 * HHH);
        const Z3 = mod(Z1 * Z2 * H);
        return new JacobianPoint(X3, Y3, Z3);
    }
    subtract(other) {
        return this.add(other.negate());
    }
    multiplyUnsafe(scalar) {
        const P0 = JacobianPoint.ZERO;
        if (typeof scalar === 'bigint' && scalar === _0n)
            return P0;
        let n = normalizeScalar(scalar);
        if (n === _1n)
            return this;
        if (!USE_ENDOMORPHISM) {
            let p = P0;
            let d = this;
            while (n > _0n) {
                if ((n & _1n) === 1n)
                    p = p.add(d);
                d = d.double();
                n >>= _1n;
            }
            return p;
        }
        let { k1neg, k1, k2neg, k2 } = endo.splitScalar(n);
        let k1p = P0;
        let k2p = P0;
        let d = this;
        while (k1 > _0n || k2 > _0n) {
            if ((k1 & _1n) === 1n)
                k1p = k1p.add(d);
            if ((k2 & _1n) === 1n)
                k2p = k2p.add(d);
            d = d.double();
            k1 >>= _1n;
            k2 >>= _1n;
        }
        if (k1neg)
            k1p = k1p.negate();
        if (k2neg)
            k2p = k2p.negate();
        k2p = new JacobianPoint(mod(k2p.x * endo.beta), k2p.y, k2p.z);
        return k1p.add(k2p);
    }
    precomputeWindow(W) {
        const windows = USE_ENDOMORPHISM ? 128 / W + 1 : 256 / W + 1;
        const points = [];
        let p = this;
        let base = p;
        for (let window = 0; window < windows; window++) {
            base = p;
            points.push(base);
            for (let i = 1; i < 2 ** (W - 1); i++) {
                base = base.add(p);
                points.push(base);
            }
            p = base.double();
        }
        return points;
    }
    wNAF(n, affinePoint) {
        if ((affinePoint == null) && this.equals(JacobianPoint.BASE))
            affinePoint = Point$1.BASE;
        const W = (affinePoint?._WINDOW_SIZE) ?? 1;
        if (256 % W === 1) {
            throw new Error('Point#wNAF: Invalid precomputation window, must be power of 2');
        }
        let precomputes = (affinePoint !== null && affinePoint !== undefined)
            ? pointPrecomputes.get(affinePoint)
            : [];
        if (precomputes === null || precomputes === undefined) {
            precomputes = this.precomputeWindow(W);
            if ((affinePoint != null) && W !== 1) {
                precomputes = JacobianPoint.normalizeZ(precomputes);
                pointPrecomputes.set(affinePoint, precomputes);
            }
        }
        let p = JacobianPoint.ZERO;
        let f = JacobianPoint.BASE;
        const windows = 1 + (USE_ENDOMORPHISM ? 128 / W : 256 / W);
        const windowSize = 2 ** (W - 1);
        const mask = BigInt(2 ** W - 1);
        const maxNumber = 2 ** W;
        const shiftBy = BigInt(W);
        for (let window = 0; window < windows; window++) {
            const offset = window * windowSize;
            let wbits = Number(n & mask);
            n >>= shiftBy;
            if (wbits > windowSize) {
                wbits -= maxNumber;
                n += _1n;
            }
            const offset1 = offset;
            const offset2 = offset + Math.abs(wbits) - 1;
            const cond1 = window % 2 !== 0;
            const cond2 = wbits < 0;
            if (wbits === 0) {
                f = f.add(constTimeNegate(cond1, precomputes[offset1]));
            }
            else {
                p = p.add(constTimeNegate(cond2, precomputes[offset2]));
            }
        }
        return { p, f };
    }
    multiply(scalar, affinePoint) {
        const n = normalizeScalar(scalar);
        let point;
        let fake;
        if (USE_ENDOMORPHISM) {
            const { k1neg, k1, k2neg, k2 } = endo.splitScalar(n);
            let { p: k1p, f: f1p } = this.wNAF(k1, affinePoint);
            let { p: k2p, f: f2p } = this.wNAF(k2, affinePoint);
            k1p = constTimeNegate(k1neg, k1p);
            k2p = constTimeNegate(k2neg, k2p);
            k2p = new JacobianPoint(mod(k2p.x * endo.beta), k2p.y, k2p.z);
            point = k1p.add(k2p);
            fake = f1p.add(f2p);
        }
        else {
            const { p, f } = this.wNAF(n, affinePoint);
            point = p;
            fake = f;
        }
        return JacobianPoint.normalizeZ([point, fake])[0];
    }
    toAffine(invZ) {
        const { x, y, z } = this;
        const is0 = this.equals(JacobianPoint.ZERO);
        if (invZ == null)
            invZ = is0 ? _8n : invert(z);
        const iz1 = invZ;
        const iz2 = mod(iz1 * iz1);
        const iz3 = mod(iz2 * iz1);
        const ax = mod(x * iz2);
        const ay = mod(y * iz3);
        const zz = mod(z * iz1);
        if (is0)
            return Point$1.ZERO;
        if (zz !== _1n)
            throw new Error('invZ was invalid');
        return new Point$1(ax, ay);
    }
}
function constTimeNegate(condition, item) {
    if (item === undefined) {
        throw new Error('Point is undefined!');
    }
    const neg = item.negate();
    return condition ? neg : item;
}
const pointPrecomputes = new WeakMap();
let Point$1 = class Point {
    constructor(x, y) {
        this.x = x;
        this.y = y;
    }
    _setWindowSize(windowSize) {
        this._WINDOW_SIZE = windowSize;
        pointPrecomputes.delete(this);
    }
    static get BASE() {
        return new Point(CURVE.Gx, CURVE.Gy);
    }
    static get ZERO() {
        return new Point(_0n, _0n);
    }
    hasEvenY() {
        return this.y % _2n === _0n;
    }
    static fromCompressedHex(bytes) {
        const isShort = bytes.length === 32;
        const x = bytesToNumber(isShort ? bytes : bytes.subarray(1));
        if (!isValidFieldElement(x))
            throw new Error('Point is not on curve');
        const y2 = weierstrass(x);
        let y = sqrtMod(y2);
        const isYOdd = (y & _1n) === _1n;
        if (isShort) {
            if (isYOdd)
                y = mod(-y);
        }
        else {
            const isFirstByteOdd = (bytes[0] & 1) === 1;
            if (isFirstByteOdd !== isYOdd)
                y = mod(-y);
        }
        const point = new Point(x, y);
        point.assertValidity();
        return point;
    }
    static fromUncompressedHex(bytes) {
        const x = bytesToNumber(bytes.subarray(1, fieldLen + 1));
        const y = bytesToNumber(bytes.subarray(fieldLen + 1, fieldLen * 2 + 1));
        const point = new Point(x, y);
        point.assertValidity();
        return point;
    }
    static fromHex(hex) {
        const bytes = ensureBytes(hex);
        const len = bytes.length;
        const header = bytes[0];
        if (len === fieldLen)
            return this.fromCompressedHex(bytes);
        if (len === compressedLen && (header === 0x02 || header === 0x03)) {
            return this.fromCompressedHex(bytes);
        }
        if (len === uncompressedLen && header === 0x04)
            return this.fromUncompressedHex(bytes);
        throw new Error(`Point.fromHex: received invalid point. Expected 32-${compressedLen} compressed bytes or ${uncompressedLen} uncompressed bytes, not ${len}`);
    }
    static fromPrivateKey(privateKey) {
        return Point.BASE.multiply(normalizePrivateKey(privateKey));
    }
    static fromSignature(msgHash, signature, recovery) {
        const { r, s } = normalizeSignature(signature);
        if (![0, 1, 2, 3].includes(recovery))
            throw new Error('Cannot recover: invalid recovery bit');
        const h = truncateHash(ensureBytes(msgHash));
        const { n } = CURVE;
        const radj = recovery === 2 || recovery === 3 ? r + n : r;
        const rinv = invert(radj, n);
        const u1 = mod(-h * rinv, n);
        const u2 = mod(s * rinv, n);
        const prefix = (recovery & 1) === 1 ? '03' : '02';
        const R = Point.fromHex(prefix + numTo32bStr(radj));
        const Q = Point.BASE.multiplyAndAddUnsafe(R, u1, u2);
        if (Q == null)
            throw new Error('Cannot recover signature: point at infinify');
        Q.assertValidity();
        return Q;
    }
    toRawBytes(isCompressed = false) {
        return hexToBytes$1(this.toHex(isCompressed));
    }
    toHex(isCompressed = false) {
        const x = numTo32bStr(this.x);
        if (isCompressed) {
            const prefix = this.hasEvenY() ? '02' : '03';
            return `${prefix}${x}`;
        }
        else {
            return `04${x}${numTo32bStr(this.y)}`;
        }
    }
    toHexX() {
        return this.toHex(true).slice(2);
    }
    toRawX() {
        return this.toRawBytes(true).slice(1);
    }
    assertValidity() {
        const msg = 'Point is not on elliptic curve';
        const { x, y } = this;
        if (!isValidFieldElement(x) || !isValidFieldElement(y))
            throw new Error(msg);
        const left = mod(y * y);
        const right = weierstrass(x);
        if (mod(left - right) !== _0n)
            throw new Error(msg);
    }
    equals(other) {
        return this.x === other.x && this.y === other.y;
    }
    negate() {
        return new Point(this.x, mod(-this.y));
    }
    double() {
        return JacobianPoint.fromAffine(this).double().toAffine();
    }
    add(other) {
        return JacobianPoint.fromAffine(this).add(JacobianPoint.fromAffine(other)).toAffine();
    }
    subtract(other) {
        return this.add(other.negate());
    }
    multiply(scalar) {
        return JacobianPoint.fromAffine(this).multiply(scalar, this).toAffine();
    }
    multiplyAndAddUnsafe(Q, a, b) {
        const P = JacobianPoint.fromAffine(this);
        const aP = a === _0n || a === _1n || this !== Point.BASE ? P.multiplyUnsafe(a) : P.multiply(a);
        const bQ = JacobianPoint.fromAffine(Q).multiplyUnsafe(b);
        const sum = aP.add(bQ);
        return sum.equals(JacobianPoint.ZERO) ? undefined : sum.toAffine();
    }
};
function sliceDER(s) {
    return Number.parseInt(s[0], 16) >= 8 ? '00' + s : s;
}
function parseDERInt(data) {
    if (data.length < 2 || data[0] !== 0x02) {
        throw new Error(`Invalid signature integer tag: ${bytesToHex$1(data)}`);
    }
    const len = data[1];
    const res = data.subarray(2, len + 2);
    if (len === 0 || res.length !== len) {
        throw new Error('Invalid signature integer: wrong length');
    }
    if (res[0] === 0x00 && res[1] <= 0x7f) {
        throw new Error('Invalid signature integer: trailing length');
    }
    return { data: bytesToNumber(res), left: data.subarray(len + 2) };
}
function parseDERSignature(data) {
    if (data.length < 2 || data[0] !== 0x30) {
        throw new Error(`Invalid signature tag: ${bytesToHex$1(data)}`);
    }
    if (data[1] !== data.length - 2) {
        throw new Error('Invalid signature: incorrect length');
    }
    const { data: r, left: sBytes } = parseDERInt(data.subarray(2));
    const { data: s, left: rBytesLeft } = parseDERInt(sBytes);
    if (rBytesLeft.length !== 0) {
        throw new Error(`Invalid signature: left bytes after parsing: ${bytesToHex$1(rBytesLeft)}`);
    }
    return { r, s };
}
class Signature {
    constructor(r, s) {
        this.r = r;
        this.s = s;
        this.assertValidity();
    }
    static fromCompact(hex) {
        const arr = hex instanceof Uint8Array;
        const name = 'Signature.fromCompact';
        if (typeof hex !== 'string' && !arr) {
            throw new TypeError(`${name}: Expected string or Uint8Array`);
        }
        const str = arr ? bytesToHex$1(hex) : hex;
        if (str.length !== 128)
            throw new Error(`${name}: Expected 64-byte hex`);
        return new Signature(hexToNumber(str.slice(0, 64)), hexToNumber(str.slice(64, 128)));
    }
    static fromDER(hex) {
        const arr = hex instanceof Uint8Array;
        if (typeof hex !== 'string' && !arr) {
            throw new TypeError('Signature.fromDER: Expected string or Uint8Array');
        }
        const { r, s } = parseDERSignature(arr ? hex : hexToBytes$1(hex));
        return new Signature(r, s);
    }
    static fromHex(hex) {
        return this.fromDER(hex);
    }
    assertValidity() {
        const { r, s } = this;
        if (!isWithinCurveOrder(r))
            throw new Error('Invalid Signature: r must be 0 < r < n');
        if (!isWithinCurveOrder(s))
            throw new Error('Invalid Signature: s must be 0 < s < n');
    }
    hasHighS() {
        const HALF = CURVE.n >> _1n;
        return this.s > HALF;
    }
    normalizeS() {
        return this.hasHighS() ? new Signature(this.r, mod(-this.s, CURVE.n)) : this;
    }
    toDERRawBytes() {
        return hexToBytes$1(this.toDERHex());
    }
    toDERHex() {
        const sHex = sliceDER(numberToHexUnpadded(this.s));
        const rHex = sliceDER(numberToHexUnpadded(this.r));
        const sHexL = sHex.length / 2;
        const rHexL = rHex.length / 2;
        const sLen = numberToHexUnpadded(sHexL);
        const rLen = numberToHexUnpadded(rHexL);
        const length = numberToHexUnpadded(rHexL + sHexL + 4);
        return `30${length}02${rLen}${rHex}02${sLen}${sHex}`;
    }
    toRawBytes() {
        return this.toDERRawBytes();
    }
    toHex() {
        return this.toDERHex();
    }
    toCompactRawBytes() {
        return hexToBytes$1(this.toCompactHex());
    }
    toCompactHex() {
        return numTo32bStr(this.r) + numTo32bStr(this.s);
    }
}
function concatBytes(...arrays) {
    if (!arrays.every((b) => b instanceof Uint8Array))
        throw new Error('Uint8Array list expected');
    if (arrays.length === 1)
        return arrays[0];
    const length = arrays.reduce((a, arr) => a + arr.length, 0);
    const result = new Uint8Array(length);
    for (let i = 0, pad = 0; i < arrays.length; i++) {
        const arr = arrays[i];
        result.set(arr, pad);
        pad += arr.length;
    }
    return result;
}
const hexes = Array.from({ length: 256 }, (_v, i) => i.toString(16).padStart(2, '0'));
function bytesToHex$1(uint8a) {
    if (!(uint8a instanceof Uint8Array))
        throw new Error('Expected Uint8Array');
    let hex = '';
    for (let i = 0; i < uint8a.length; i++) {
        hex += hexes[uint8a[i]];
    }
    return hex;
}
const POW_2_256 = BigInt('0x10000000000000000000000000000000000000000000000000000000000000000');
function numTo32bStr(num) {
    if (typeof num !== 'bigint')
        throw new Error('Expected bigint');
    if (!(_0n <= num && num < POW_2_256))
        throw new Error('Expected number 0 <= n < 2^256');
    return num.toString(16).padStart(64, '0');
}
function numTo32b(num) {
    const b = hexToBytes$1(numTo32bStr(num));
    if (b.length !== 32)
        throw new Error('Error: expected 32 bytes');
    return b;
}
function numberToHexUnpadded(num) {
    const hex = num.toString(16);
    return (hex.length & 1) !== 0 ? `0${hex}` : hex;
}
function hexToNumber(hex) {
    if (typeof hex !== 'string') {
        throw new TypeError('hexToNumber: expected string, got ' + typeof hex);
    }
    return BigInt(`0x${hex}`);
}
function hexToBytes$1(hex) {
    if (typeof hex !== 'string') {
        throw new TypeError('hexToBytes: expected string, got ' + typeof hex);
    }
    if (hex.length % 2 !== 0)
        throw new Error('hexToBytes: received invalid unpadded hex' + String(hex.length));
    const array = new Uint8Array(hex.length / 2);
    for (let i = 0; i < array.length; i++) {
        const j = i * 2;
        const hexByte = hex.slice(j, j + 2);
        const byte = Number.parseInt(hexByte, 16);
        if (Number.isNaN(byte) || byte < 0)
            throw new Error('Invalid byte sequence');
        array[i] = byte;
    }
    return array;
}
function bytesToNumber(bytes) {
    return hexToNumber(bytesToHex$1(bytes));
}
function ensureBytes(hex) {
    return hex instanceof Uint8Array ? Uint8Array.from(hex) : hexToBytes$1(hex);
}
function normalizeScalar(num) {
    if (typeof num === 'number' && Number.isSafeInteger(num) && num > 0)
        return BigInt(num);
    if (typeof num === 'bigint' && isWithinCurveOrder(num))
        return num;
    throw new TypeError('Expected valid private scalar: 0 < scalar < curve.n');
}
function mod(a, b = CURVE.P) {
    const result = a % b;
    return result >= _0n ? result : b + result;
}
function pow2(x, power) {
    const { P } = CURVE;
    let res = x;
    while (power-- > _0n) {
        res *= res;
        res %= P;
    }
    return res;
}
function sqrtMod(x) {
    const { P } = CURVE;
    const _6n = BigInt(6);
    const _11n = BigInt(11);
    const _22n = BigInt(22);
    const _23n = BigInt(23);
    const _44n = BigInt(44);
    const _88n = BigInt(88);
    const b2 = (x * x * x) % P;
    const b3 = (b2 * b2 * x) % P;
    const b6 = (pow2(b3, _3n) * b3) % P;
    const b9 = (pow2(b6, _3n) * b3) % P;
    const b11 = (pow2(b9, _2n) * b2) % P;
    const b22 = (pow2(b11, _11n) * b11) % P;
    const b44 = (pow2(b22, _22n) * b22) % P;
    const b88 = (pow2(b44, _44n) * b44) % P;
    const b176 = (pow2(b88, _88n) * b88) % P;
    const b220 = (pow2(b176, _44n) * b44) % P;
    const b223 = (pow2(b220, _3n) * b3) % P;
    const t1 = (pow2(b223, _23n) * b22) % P;
    const t2 = (pow2(t1, _6n) * b2) % P;
    const rt = pow2(t2, _2n);
    const xc = (rt * rt) % P;
    if (xc !== x)
        throw new Error('Cannot find square root');
    return rt;
}
function invert(number, modulo = CURVE.P) {
    if (number === _0n || modulo <= _0n) {
        throw new Error(`invert: expected positive integers, got n=${number} mod=${modulo}`);
    }
    let a = mod(number, modulo);
    let b = modulo;
    let x = _0n, u = _1n;
    while (a !== _0n) {
        const q = b / a;
        const r = b % a;
        const m = x - u * q;
        b = a, a = r, x = u, u = m;
    }
    const gcd = b;
    if (gcd !== _1n)
        throw new Error('invert: does not exist');
    return mod(x, modulo);
}
function invertBatch(nums, p = CURVE.P) {
    const scratch = new Array(nums.length);
    const lastMultiplied = nums.reduce((acc, num, i) => {
        if (num === _0n)
            return acc;
        scratch[i] = acc;
        return mod(acc * num, p);
    }, _1n);
    const inverted = invert(lastMultiplied, p);
    nums.reduceRight((acc, num, i) => {
        if (num === _0n)
            return acc;
        scratch[i] = mod(acc * scratch[i], p);
        return mod(acc * num, p);
    }, inverted);
    return scratch;
}
function bits2int_2(bytes) {
    const delta = bytes.length * 8 - groupLen * 8;
    const num = bytesToNumber(bytes);
    return delta > 0 ? num >> BigInt(delta) : num;
}
function truncateHash(hash, truncateOnly = false) {
    const h = bits2int_2(hash);
    if (truncateOnly)
        return h;
    const { n } = CURVE;
    return h >= n ? h - n : h;
}
let _sha256Sync;
let _hmacSha256Sync;
class HmacDrbg {
    constructor(hashLen, qByteLen) {
        this.hashLen = hashLen;
        this.qByteLen = qByteLen;
        if (typeof hashLen !== 'number' || hashLen < 2)
            throw new Error('hashLen must be a number');
        if (typeof qByteLen !== 'number' || qByteLen < 2)
            throw new Error('qByteLen must be a number');
        this.v = new Uint8Array(hashLen).fill(1);
        this.k = new Uint8Array(hashLen).fill(0);
        this.counter = 0;
    }
    async hmac(...values) {
        return utils.hmacSha256(this.k, ...values);
    }
    hmacSync(...values) {
        if (_hmacSha256Sync === undefined) {
            throw new Error('hmacSha256Sync is undefined!');
        }
        return _hmacSha256Sync(this.k, ...values);
    }
    checkSync() {
        if (typeof _hmacSha256Sync !== 'function')
            throw new ShaError('hmacSha256Sync needs to be set');
    }
    incr() {
        if (this.counter >= 1000)
            throw new Error('Tried 1,000 k values for sign(), all were invalid');
        this.counter += 1;
    }
    async reseed(seed = new Uint8Array()) {
        this.k = await this.hmac(this.v, Uint8Array.from([0x00]), seed);
        this.v = await this.hmac(this.v);
        if (seed.length === 0)
            return;
        this.k = await this.hmac(this.v, Uint8Array.from([0x01]), seed);
        this.v = await this.hmac(this.v);
    }
    reseedSync(seed = new Uint8Array()) {
        this.checkSync();
        this.k = this.hmacSync(this.v, Uint8Array.from([0x00]), seed);
        this.v = this.hmacSync(this.v);
        if (seed.length === 0)
            return;
        this.k = this.hmacSync(this.v, Uint8Array.from([0x01]), seed);
        this.v = this.hmacSync(this.v);
    }
    async generate() {
        this.incr();
        let len = 0;
        const out = [];
        while (len < this.qByteLen) {
            this.v = await this.hmac(this.v);
            const sl = this.v.slice();
            out.push(sl);
            len += this.v.length;
        }
        return concatBytes(...out);
    }
    generateSync() {
        this.checkSync();
        this.incr();
        let len = 0;
        const out = [];
        while (len < this.qByteLen) {
            this.v = this.hmacSync(this.v);
            const sl = this.v.slice();
            out.push(sl);
            len += this.v.length;
        }
        return concatBytes(...out);
    }
}
function isWithinCurveOrder(num) {
    return _0n < num && num < CURVE.n;
}
function isValidFieldElement(num) {
    return _0n < num && num < CURVE.P;
}
function kmdToSig(kBytes, m, d, lowS = true) {
    const { n } = CURVE;
    const k = truncateHash(kBytes, true);
    if (!isWithinCurveOrder(k))
        return;
    const kinv = invert(k, n);
    const q = Point$1.BASE.multiply(k);
    const r = mod(q.x, n);
    if (r === _0n)
        return;
    const s = mod(kinv * mod(m + d * r, n), n);
    if (s === _0n)
        return;
    let sig = new Signature(r, s);
    let recovery = (q.x === sig.r ? 0 : 2) | Number(q.y & _1n);
    if (lowS && sig.hasHighS()) {
        sig = sig.normalizeS();
        recovery ^= 1;
    }
    return { sig, recovery };
}
function normalizePrivateKey(key) {
    let num;
    if (typeof key === 'bigint') {
        num = key;
    }
    else if (typeof key === 'number' && Number.isSafeInteger(key) && key > 0) {
        num = BigInt(key);
    }
    else if (typeof key === 'string') {
        if (key.length !== 2 * groupLen)
            throw new Error('Expected 32 bytes of private key');
        num = hexToNumber(key);
    }
    else if (key instanceof Uint8Array) {
        if (key.length !== groupLen)
            throw new Error('Expected 32 bytes of private key');
        num = bytesToNumber(key);
    }
    else {
        throw new TypeError('Expected valid private key');
    }
    if (!isWithinCurveOrder(num))
        throw new Error('Expected private key: 0 < key < n');
    return num;
}
function normalizePublicKey(publicKey) {
    if (publicKey instanceof Point$1) {
        publicKey.assertValidity();
        return publicKey;
    }
    else {
        return Point$1.fromHex(publicKey);
    }
}
function normalizeSignature(signature) {
    if (signature instanceof Signature) {
        signature.assertValidity();
        return signature;
    }
    try {
        return Signature.fromDER(signature);
    }
    catch (error) {
        return Signature.fromCompact(signature);
    }
}
function getPublicKey(privateKey, isCompressed = false) {
    return Point$1.fromPrivateKey(privateKey).toRawBytes(isCompressed);
}
function recoverPublicKey(msgHash, signature, recovery, isCompressed = false) {
    return Point$1.fromSignature(msgHash, signature, recovery).toRawBytes(isCompressed);
}
function isProbPub(item) {
    const arr = item instanceof Uint8Array;
    const str = typeof item === 'string';
    const len = (arr || str) && item.length;
    if (arr)
        return len === compressedLen || len === uncompressedLen;
    if (str)
        return len === compressedLen * 2 || len === uncompressedLen * 2;
    if (item instanceof Point$1)
        return true;
    return false;
}
function getSharedSecret(privateA, publicB, isCompressed = false) {
    if (isProbPub(privateA))
        throw new TypeError('getSharedSecret: first arg must be private key');
    if (!isProbPub(publicB))
        throw new TypeError('getSharedSecret: second arg must be public key');
    const b = normalizePublicKey(publicB);
    b.assertValidity();
    return b.multiply(normalizePrivateKey(privateA)).toRawBytes(isCompressed);
}
function bits2int(bytes) {
    const slice = bytes.length > fieldLen ? bytes.slice(0, fieldLen) : bytes;
    return bytesToNumber(slice);
}
function bits2octets(bytes) {
    const z1 = bits2int(bytes);
    const z2 = mod(z1, CURVE.n);
    return int2octets(z2 < _0n ? z1 : z2);
}
function int2octets(num) {
    return numTo32b(num);
}
function initSigArgs(msgHash, privateKey, extraEntropy) {
    if (msgHash === null)
        throw new Error('sign: expected valid message hash, not null.');
    const h1 = ensureBytes(msgHash);
    const d = normalizePrivateKey(privateKey);
    const seedArgs = [int2octets(d), bits2octets(h1)];
    if (extraEntropy != null) {
        if (extraEntropy === true)
            extraEntropy = utils.randomBytes(fieldLen);
        const e = ensureBytes(extraEntropy);
        if (e.length !== fieldLen)
            throw new Error(`sign: Expected ${fieldLen} bytes of extra data`);
        seedArgs.push(e);
    }
    const seed = concatBytes(...seedArgs);
    const m = bits2int(h1);
    return { seed, m, d };
}
function finalizeSig(recSig, opts) {
    const { sig, recovery } = recSig;
    const { der, recovered } = Object.assign({ canonical: true, der: true }, opts);
    const hashed = der ? sig.toDERRawBytes() : sig.toCompactRawBytes();
    return recovered === true ? [hashed, recovery] : hashed;
}
async function sign$1(msgHash, privKey, opts = {}) {
    const { seed, m, d } = initSigArgs(msgHash, privKey, opts.extraEntropy);
    const drbg = new HmacDrbg(hashLen, groupLen);
    await drbg.reseed(seed);
    let sig;
    while ((sig = kmdToSig(await drbg.generate(), m, d, opts.canonical)) == null)
        await drbg.reseed();
    return finalizeSig(sig, opts);
}
function signSync(msgHash, privKey, opts = {}) {
    const { seed, m, d } = initSigArgs(msgHash, privKey, opts.extraEntropy);
    const drbg = new HmacDrbg(hashLen, groupLen);
    drbg.reseedSync(seed);
    let sig;
    while ((sig = kmdToSig(drbg.generateSync(), m, d, opts.canonical)) == null)
        drbg.reseedSync();
    return finalizeSig(sig, opts);
}
const vopts = { strict: true };
function verify$1(signature, msgHash, publicKey, opts = vopts) {
    let sig;
    try {
        sig = normalizeSignature(signature);
        msgHash = ensureBytes(msgHash);
    }
    catch (error) {
        return false;
    }
    const { r, s } = sig;
    if (opts.strict === true && sig.hasHighS())
        return false;
    const h = truncateHash(msgHash);
    let P;
    try {
        P = normalizePublicKey(publicKey);
    }
    catch (error) {
        return false;
    }
    const { n } = CURVE;
    const sinv = invert(s, n);
    const u1 = mod(h * sinv, n);
    const u2 = mod(r * sinv, n);
    const R = Point$1.BASE.multiplyAndAddUnsafe(P, u1, u2);
    if (R == null)
        return false;
    const v = mod(R.x, n);
    return v === r;
}
function schnorrChallengeFinalize(ch) {
    return mod(bytesToNumber(ch), CURVE.n);
}
class SchnorrSignature {
    constructor(r, s) {
        this.r = r;
        this.s = s;
        this.assertValidity();
    }
    static fromHex(hex) {
        const bytes = ensureBytes(hex);
        if (bytes.length !== 64) {
            throw new TypeError(`SchnorrSignature.fromHex: expected 64 bytes, not ${bytes.length}`);
        }
        const r = bytesToNumber(bytes.subarray(0, 32));
        const s = bytesToNumber(bytes.subarray(32, 64));
        return new SchnorrSignature(r, s);
    }
    assertValidity() {
        const { r, s } = this;
        if (!isValidFieldElement(r) || !isWithinCurveOrder(s))
            throw new Error('Invalid signature');
    }
    toHex() {
        return numTo32bStr(this.r) + numTo32bStr(this.s);
    }
    toRawBytes() {
        return hexToBytes$1(this.toHex());
    }
}
function schnorrGetPublicKey(privateKey) {
    return Point$1.fromPrivateKey(privateKey).toRawX();
}
class InternalSchnorrSignature {
    constructor(message, privateKey, auxRand = utils.randomBytes()) {
        if (message === null)
            throw new TypeError('sign: Expected valid message, not null.');
        this.m = ensureBytes(message);
        const { x, scalar } = this.getScalar(normalizePrivateKey(privateKey));
        this.px = x;
        this.d = scalar;
        this.rand = ensureBytes(auxRand);
        if (this.rand.length !== 32)
            throw new TypeError('sign: Expected 32 bytes of aux randomness');
    }
    getScalar(priv) {
        const point = Point$1.fromPrivateKey(priv);
        const scalar = point.hasEvenY() ? priv : CURVE.n - priv;
        return { point, scalar, x: point.toRawX() };
    }
    initNonce(d, t0h) {
        return numTo32b(d ^ bytesToNumber(t0h));
    }
    finalizeNonce(k0h) {
        const k0 = mod(bytesToNumber(k0h), CURVE.n);
        if (k0 === _0n)
            throw new Error('sign: Creation of signature failed. k is zero');
        const { point: R, x: rx, scalar: k } = this.getScalar(k0);
        return { R, rx, k };
    }
    finalizeSig(R, k, e, d) {
        return new SchnorrSignature(R.x, mod(k + e * d, CURVE.n)).toRawBytes();
    }
    error() {
        throw new Error('sign: Invalid signature produced');
    }
    async calc() {
        const { m, d, px, rand } = this;
        const tag = utils.taggedHash;
        const t = this.initNonce(d, await tag(TAGS.aux, rand));
        const { R, rx, k } = this.finalizeNonce(await tag(TAGS.nonce, t, px, m));
        const e = schnorrChallengeFinalize(await tag(TAGS.challenge, rx, px, m));
        const sig = this.finalizeSig(R, k, e, d);
        if (!(await schnorrVerify(sig, m, px)))
            this.error();
        return sig;
    }
    calcSync() {
        const { m, d, px, rand } = this;
        const tag = utils.taggedHashSync;
        const t = this.initNonce(d, tag(TAGS.aux, rand));
        const { R, rx, k } = this.finalizeNonce(tag(TAGS.nonce, t, px, m));
        const e = schnorrChallengeFinalize(tag(TAGS.challenge, rx, px, m));
        const sig = this.finalizeSig(R, k, e, d);
        if (!schnorrVerifySync(sig, m, px))
            this.error();
        return sig;
    }
}
async function schnorrSign(msg, privKey, auxRand) {
    return new InternalSchnorrSignature(msg, privKey, auxRand).calc();
}
function schnorrSignSync(msg, privKey, auxRand) {
    return new InternalSchnorrSignature(msg, privKey, auxRand).calcSync();
}
function initSchnorrVerify(signature, message, publicKey) {
    const raw = signature instanceof SchnorrSignature;
    const sig = raw ? signature : SchnorrSignature.fromHex(signature);
    if (raw)
        sig.assertValidity();
    return {
        ...sig,
        m: ensureBytes(message),
        P: normalizePublicKey(publicKey)
    };
}
function finalizeSchnorrVerify(r, P, s, e) {
    const R = Point$1.BASE.multiplyAndAddUnsafe(P, normalizePrivateKey(s), mod(-e, CURVE.n));
    if ((R == null) || !R.hasEvenY() || R.x !== r)
        return false;
    return true;
}
async function schnorrVerify(signature, message, publicKey) {
    try {
        const { r, s, m, P } = initSchnorrVerify(signature, message, publicKey);
        const e = schnorrChallengeFinalize(await utils.taggedHash(TAGS.challenge, numTo32b(r), P.toRawX(), m));
        return finalizeSchnorrVerify(r, P, s, e);
    }
    catch (error) {
        return false;
    }
}
function schnorrVerifySync(signature, message, publicKey) {
    try {
        const { r, s, m, P } = initSchnorrVerify(signature, message, publicKey);
        const e = schnorrChallengeFinalize(utils.taggedHashSync(TAGS.challenge, numTo32b(r), P.toRawX(), m));
        return finalizeSchnorrVerify(r, P, s, e);
    }
    catch (error) {
        if (error instanceof ShaError)
            throw error;
        return false;
    }
}
const schnorr = {
    Signature: SchnorrSignature,
    getPublicKey: schnorrGetPublicKey,
    sign: schnorrSign,
    verify: schnorrVerify,
    signSync: schnorrSignSync,
    verifySync: schnorrVerifySync
};
Point$1.BASE._setWindowSize(8);
const TAGS = {
    challenge: 'BIP0340/challenge',
    aux: 'BIP0340/aux',
    nonce: 'BIP0340/nonce'
};
const TAGGED_HASH_PREFIXES = {};
const utils = {
    bytesToHex: bytesToHex$1,
    hexToBytes: hexToBytes$1,
    concatBytes,
    mod,
    invert,
    isValidPrivateKey(privateKey) {
        try {
            normalizePrivateKey(privateKey);
            return true;
        }
        catch (error) {
            return false;
        }
    },
    _bigintTo32Bytes: numTo32b,
    _normalizePrivateKey: normalizePrivateKey,
    hashToPrivateKey: (hash) => {
        hash = ensureBytes(hash);
        const minLen = groupLen + 8;
        if (hash.length < minLen || hash.length > 1024) {
            throw new Error('Expected valid bytes of private key as per FIPS 186');
        }
        const num = mod(bytesToNumber(hash), CURVE.n - _1n) + _1n;
        return numTo32b(num);
    },
    randomBytes: (bytesLength = 32) => {
        return webcrypto$2.getRandomValues(new Uint8Array(bytesLength));
    },
    randomPrivateKey: () => utils.hashToPrivateKey(utils.randomBytes(groupLen + 8)),
    precompute(windowSize = 8, point = Point$1.BASE) {
        const cached = point === Point$1.BASE ? point : new Point$1(point.x, point.y);
        cached._setWindowSize(windowSize);
        cached.multiply(_3n);
        return cached;
    },
    sha256: async (...messages) => {
        const buffer = await webcrypto$2.subtle.digest('SHA-256', concatBytes(...messages));
        return new Uint8Array(buffer);
    },
    hmacSha256: async (key, ...messages) => {
        const ckey = await webcrypto$2.subtle.importKey('raw', key, { name: 'HMAC', hash: { name: 'SHA-256' } }, false, ['sign']);
        const message = concatBytes(...messages);
        const buffer = await webcrypto$2.subtle.sign('HMAC', ckey, message);
        return new Uint8Array(buffer);
    },
    sha256Sync: undefined,
    hmacSha256Sync: undefined,
    taggedHash: async (tag, ...messages) => {
        let tagP = TAGGED_HASH_PREFIXES[tag];
        if (tagP === undefined) {
            const tagH = await utils.sha256(Uint8Array.from(tag, (c) => c.charCodeAt(0)));
            tagP = concatBytes(tagH, tagH);
            TAGGED_HASH_PREFIXES[tag] = tagP;
        }
        return utils.sha256(tagP, ...messages);
    },
    taggedHashSync: (tag, ...messages) => {
        if (typeof _sha256Sync !== 'function') {
            throw new ShaError('sha256Sync is undefined, you need to set it');
        }
        let tagP = TAGGED_HASH_PREFIXES[tag];
        if (tagP === undefined) {
            const tagH = _sha256Sync(Uint8Array.from(tag, (c) => c.charCodeAt(0)));
            tagP = concatBytes(tagH, tagH);
            TAGGED_HASH_PREFIXES[tag] = tagP;
        }
        return _sha256Sync(tagP, ...messages);
    },
    _JacobianPoint: JacobianPoint
};
Object.defineProperties(utils, {
    sha256Sync: {
        configurable: false,
        get() {
            return _sha256Sync;
        },
        set(val) {
            if (_sha256Sync == null)
                _sha256Sync = val;
        }
    },
    hmacSha256Sync: {
        configurable: false,
        get() {
            return _hmacSha256Sync;
        },
        set(val) {
            if (_hmacSha256Sync == null)
                _hmacSha256Sync = val;
        }
    }
});

var module$1 = /*#__PURE__*/Object.freeze({
    __proto__: null,
    CURVE: CURVE,
    Point: Point$1,
    Signature: Signature,
    getPublicKey: getPublicKey,
    getSharedSecret: getSharedSecret,
    recoverPublicKey: recoverPublicKey,
    schnorr: schnorr,
    sign: sign$1,
    signSync: signSync,
    utils: utils,
    verify: verify$1
});

const webcrypto$1 = getCryptoModule$1();
function getCryptoModule$1() {
    let webcrypto = null;
    if (exists$1(crypto.webcrypto)) {
        webcrypto = crypto.webcrypto;
    }
    else if (exists$1(globalThis.crypto)) {
        webcrypto = globalThis.crypto;
    }
    else if (exists$1(window.crypto)) {
        webcrypto = window.crypto;
    }
    if (!exists$1(webcrypto)) {
        console.log('Search for webcrypto library failed!');
        console.log('crypto:', typeof crypto.webcrypto);
        console.log('globalThis.crypto:', typeof globalThis?.crypto);
        console.log('window.crypto:', typeof window?.crypto);
        throw new Error('Unable to find webcrypto library. If you are running in a NodeJs environment, try using version 19 or newer. If you are running in a browser environment, make sure that you are running in a secure context.');
    }
    if (!exists$1(webcrypto?.subtle)) {
        throw new Error('Webcrypto library is partially disabled! Make sure you are running within a secure environment. If you are getting this error in the browser, make sure https is enabled and/or you are testing locally using 127.0.0.1.');
    }
    return webcrypto;
}
function exists$1(x) {
    try {
        return (typeof x !== 'undefined' && x !== null);
    }
    catch {
        return false;
    }
}

function joinArray(arr) {
    let i, idx = 0;
    const size = arr.reduce((prev, curr) => prev + curr.length, 0);
    const buff = new Uint8Array(size);
    for (const bytes of arr) {
        for (i = 0; i < bytes.length; idx++, i++) {
            buff[idx] = bytes[i];
        }
    }
    return buff;
}

const digestLength = 32;
const blockSize = 64;
const K = new Uint32Array([
    0x428a2f98, 0x71374491, 0xb5c0fbcf, 0xe9b5dba5, 0x3956c25b,
    0x59f111f1, 0x923f82a4, 0xab1c5ed5, 0xd807aa98, 0x12835b01,
    0x243185be, 0x550c7dc3, 0x72be5d74, 0x80deb1fe, 0x9bdc06a7,
    0xc19bf174, 0xe49b69c1, 0xefbe4786, 0x0fc19dc6, 0x240ca1cc,
    0x2de92c6f, 0x4a7484aa, 0x5cb0a9dc, 0x76f988da, 0x983e5152,
    0xa831c66d, 0xb00327c8, 0xbf597fc7, 0xc6e00bf3, 0xd5a79147,
    0x06ca6351, 0x14292967, 0x27b70a85, 0x2e1b2138, 0x4d2c6dfc,
    0x53380d13, 0x650a7354, 0x766a0abb, 0x81c2c92e, 0x92722c85,
    0xa2bfe8a1, 0xa81a664b, 0xc24b8b70, 0xc76c51a3, 0xd192e819,
    0xd6990624, 0xf40e3585, 0x106aa070, 0x19a4c116, 0x1e376c08,
    0x2748774c, 0x34b0bcb5, 0x391c0cb3, 0x4ed8aa4a, 0x5b9cca4f,
    0x682e6ff3, 0x748f82ee, 0x78a5636f, 0x84c87814, 0x8cc70208,
    0x90befffa, 0xa4506ceb, 0xbef9a3f7, 0xc67178f2
]);
function hashBlocks(w, v, p, pos, len) {
    let a, b, c, d, e, f, g, h, u, i, j, t1, t2;
    while (len >= 64) {
        a = v[0];
        b = v[1];
        c = v[2];
        d = v[3];
        e = v[4];
        f = v[5];
        g = v[6];
        h = v[7];
        for (i = 0; i < 16; i++) {
            j = pos + i * 4;
            w[i] = (((p[j] & 0xff) << 24) | ((p[j + 1] & 0xff) << 16) |
                ((p[j + 2] & 0xff) << 8) | (p[j + 3] & 0xff));
        }
        for (i = 16; i < 64; i++) {
            u = w[i - 2];
            t1 = (u >>> 17 | u << (32 - 17)) ^ (u >>> 19 | u << (32 - 19)) ^ (u >>> 10);
            u = w[i - 15];
            t2 = (u >>> 7 | u << (32 - 7)) ^ (u >>> 18 | u << (32 - 18)) ^ (u >>> 3);
            w[i] = (t1 + w[i - 7] | 0) + (t2 + w[i - 16] | 0);
        }
        for (i = 0; i < 64; i++) {
            t1 = (((((e >>> 6 | e << (32 - 6)) ^ (e >>> 11 | e << (32 - 11)) ^
                (e >>> 25 | e << (32 - 25))) + ((e & f) ^ (~e & g))) | 0) +
                ((h + ((K[i] + w[i]) | 0)) | 0)) | 0;
            t2 = (((a >>> 2 | a << (32 - 2)) ^ (a >>> 13 | a << (32 - 13)) ^
                (a >>> 22 | a << (32 - 22))) + ((a & b) ^ (a & c) ^ (b & c))) | 0;
            h = g;
            g = f;
            f = e;
            e = (d + t1) | 0;
            d = c;
            c = b;
            b = a;
            a = (t1 + t2) | 0;
        }
        v[0] += a;
        v[1] += b;
        v[2] += c;
        v[3] += d;
        v[4] += e;
        v[5] += f;
        v[6] += g;
        v[7] += h;
        pos += 64;
        len -= 64;
    }
    return pos;
}
let Hash$1 = class Hash {
    constructor() {
        this.digestLength = digestLength;
        this.blockSize = blockSize;
        this.state = new Int32Array(8);
        this.temp = new Int32Array(64);
        this.buffer = new Uint8Array(128);
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        this.reset();
    }
    reset() {
        this.state[0] = 0x6a09e667;
        this.state[1] = 0xbb67ae85;
        this.state[2] = 0x3c6ef372;
        this.state[3] = 0xa54ff53a;
        this.state[4] = 0x510e527f;
        this.state[5] = 0x9b05688c;
        this.state[6] = 0x1f83d9ab;
        this.state[7] = 0x5be0cd19;
        this.bufferLength = 0;
        this.bytesHashed = 0;
        this.finished = false;
        return this;
    }
    clean() {
        for (let i = 0; i < this.buffer.length; i++) {
            this.buffer[i] = 0;
        }
        for (let i = 0; i < this.temp.length; i++) {
            this.temp[i] = 0;
        }
        this.reset();
    }
    update(data, dataLength = data.length) {
        if (this.finished) {
            throw new Error("SHA256: can't update because hash was finished.");
        }
        let dataPos = 0;
        this.bytesHashed += dataLength;
        if (this.bufferLength > 0) {
            while (this.bufferLength < 64 && dataLength > 0) {
                this.buffer[this.bufferLength++] = data[dataPos++];
                dataLength--;
            }
            if (this.bufferLength === 64) {
                hashBlocks(this.temp, this.state, this.buffer, 0, 64);
                this.bufferLength = 0;
            }
        }
        if (dataLength >= 64) {
            dataPos = hashBlocks(this.temp, this.state, data, dataPos, dataLength);
            dataLength %= 64;
        }
        while (dataLength > 0) {
            this.buffer[this.bufferLength++] = data[dataPos++];
            dataLength--;
        }
        return this;
    }
    finish(out) {
        if (!this.finished) {
            const bytesHashed = this.bytesHashed;
            const left = this.bufferLength;
            const bitLenHi = (bytesHashed / 0x20000000) | 0;
            const bitLenLo = bytesHashed << 3;
            const padLength = (bytesHashed % 64 < 56) ? 64 : 128;
            this.buffer[left] = 0x80;
            for (let i = left + 1; i < padLength - 8; i++) {
                this.buffer[i] = 0;
            }
            this.buffer[padLength - 8] = (bitLenHi >>> 24) & 0xff;
            this.buffer[padLength - 7] = (bitLenHi >>> 16) & 0xff;
            this.buffer[padLength - 6] = (bitLenHi >>> 8) & 0xff;
            this.buffer[padLength - 5] = (bitLenHi >>> 0) & 0xff;
            this.buffer[padLength - 4] = (bitLenLo >>> 24) & 0xff;
            this.buffer[padLength - 3] = (bitLenLo >>> 16) & 0xff;
            this.buffer[padLength - 2] = (bitLenLo >>> 8) & 0xff;
            this.buffer[padLength - 1] = (bitLenLo >>> 0) & 0xff;
            hashBlocks(this.temp, this.state, this.buffer, 0, padLength);
            this.finished = true;
        }
        for (let i = 0; i < 8; i++) {
            out[i * 4 + 0] = (this.state[i] >>> 24) & 0xff;
            out[i * 4 + 1] = (this.state[i] >>> 16) & 0xff;
            out[i * 4 + 2] = (this.state[i] >>> 8) & 0xff;
            out[i * 4 + 3] = (this.state[i] >>> 0) & 0xff;
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
    _saveState(out) {
        for (let i = 0; i < this.state.length; i++) {
            out[i] = this.state[i];
        }
    }
    _restoreState(from, bytesHashed) {
        for (let i = 0; i < this.state.length; i++) {
            this.state[i] = from[i];
        }
        this.bytesHashed = bytesHashed;
        this.finished = false;
        this.bufferLength = 0;
    }
};
class HMAC {
    constructor(key) {
        this.inner = new Hash$1();
        this.outer = new Hash$1();
        this.blockSize = this.inner.blockSize;
        this.digestLength = this.inner.digestLength;
        const pad = new Uint8Array(this.blockSize);
        if (key.length > this.blockSize) {
            (new Hash$1()).update(key).finish(pad).clean();
        }
        else {
            for (let i = 0; i < key.length; i++) {
                pad[i] = key[i];
            }
        }
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36;
        }
        this.inner.update(pad);
        for (let i = 0; i < pad.length; i++) {
            pad[i] ^= 0x36 ^ 0x5c;
        }
        this.outer.update(pad);
        this.istate = new Uint32Array(8);
        this.ostate = new Uint32Array(8);
        this.inner._saveState(this.istate);
        this.outer._saveState(this.ostate);
        for (let i = 0; i < pad.length; i++) {
            pad[i] = 0;
        }
    }
    reset() {
        this.inner._restoreState(this.istate, this.inner.blockSize);
        this.outer._restoreState(this.ostate, this.outer.blockSize);
        return this;
    }
    clean() {
        for (let i = 0; i < this.istate.length; i++) {
            this.ostate[i] = this.istate[i] = 0;
        }
        this.inner.clean();
        this.outer.clean();
    }
    update(data) {
        this.inner.update(data);
        return this;
    }
    finish(out) {
        if (this.outer.finished) {
            this.outer.finish(out);
        }
        else {
            this.inner.finish(out);
            this.outer.update(out, this.digestLength).finish(out);
        }
        return this;
    }
    digest() {
        const out = new Uint8Array(this.digestLength);
        this.finish(out);
        return out;
    }
}
function sha256$1(data) {
    const h = (new Hash$1()).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}
function hash256$1(data) {
    return sha256$1(sha256$1(data));
}
function hmac256$1(key, data) {
    const h = (new HMAC(key)).update(data);
    const digest = h.digest();
    h.clean();
    return digest;
}

const ec$2 = new TextEncoder();
const ALPHABETS = [
    {
        name: 'base58',
        charset: '123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz'
    }
];
function getAlphabet(name) {
    for (const alpha of ALPHABETS) {
        if (alpha.name === name) {
            return alpha.charset;
        }
    }
    throw TypeError('Charset does not exist: ' + name);
}
function encode$1(data, charset, padding = false) {
    if (typeof data === 'string')
        data = ec$2.encode(data);
    const alphabet = getAlphabet(charset);
    const len = alphabet.length;
    const d = [];
    let s = '', i, j = 0, c, n;
    for (i = 0; i < data.length; i++) {
        j = 0;
        c = data[i];
        s += (c > 0 || (s.length ^ i) > 0) ? '' : '1';
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * 256 + c : c;
            c = n / len | 0;
            d[j] = n % len;
            j++;
        }
    }
    while (j-- > 0) {
        s += alphabet[d[j]];
    }
    return (padding && s.length % 4 > 0)
        ? s + '='.repeat(4 - s.length % 4)
        : s;
}
function decode$1(encoded, charset) {
    const alphabet = getAlphabet(charset);
    const len = alphabet.length, d = [], b = [];
    encoded = encoded.replace('=', '');
    let i, j = 0, c, n;
    for (i = 0; i < encoded.length; i++) {
        j = 0;
        c = alphabet.indexOf(encoded[i]);
        if (c < 0) {
            throw new Error(`Character range out of bounds: ${c}`);
        }
        if (!(c > 0 || (b.length ^ i) > 0))
            b.push(0);
        while (j in d || c > 0) {
            n = d[j];
            n = n > 0 ? n * len + c : c;
            c = n >> 8;
            d[j] = n % 256;
            j++;
        }
    }
    while (j-- > 0) {
        b.push(d[j]);
    }
    return new Uint8Array(b);
}
function addChecksum(data) {
    const sum = hash256$1(data);
    return joinArray([data, sum.slice(0, 4)]);
}
function checkTheSum(data) {
    const ret = data.slice(0, -4);
    const chk = data.slice(-4);
    const sum = hash256$1(ret).slice(0, 4);
    if (sum.toString() !== chk.toString()) {
        throw new Error('Invalid checksum!');
    }
    return ret;
}
const BaseX = {
    encode: encode$1,
    decode: decode$1
};
const Base58C = {
    encode: (data) => {
        const withSum = addChecksum(data);
        return BaseX.encode(withSum, 'base58');
    },
    decode: (data) => {
        const decoded = BaseX.decode(data, 'base58');
        return checkTheSum(decoded);
    }
};

const CHARSET = 'qpzry9x8gf2tvdw0s3jn54khce6mua7l';
const GENERATOR = [0x3b6a57b2, 0x26508e6d, 0x1ea119fa, 0x3d4233dd, 0x2a1462b3];
const ENCODINGS = [
    { version: 0, name: 'bech32', const: 1 },
    { version: 1, name: 'bech32m', const: 0x2bc830a3 }
];
function polymod(values) {
    let chk = 1;
    for (let p = 0; p < values.length; ++p) {
        const top = chk >> 25;
        chk = (chk & 0x1ffffff) << 5 ^ values[p];
        for (let i = 0; i < 5; ++i) {
            if (((top >> i) & 1) !== 0) {
                chk ^= GENERATOR[i];
            }
        }
    }
    return chk;
}
function hrpExpand(hrp) {
    const ret = [];
    let p;
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) >> 5);
    }
    ret.push(0);
    for (p = 0; p < hrp.length; ++p) {
        ret.push(hrp.charCodeAt(p) & 31);
    }
    return ret;
}
function verifyChecksum(hrp, data, enc) {
    const combined = hrpExpand(hrp).concat(data);
    return polymod(combined) === enc.const;
}
function createChecksum(hrp, data, enc) {
    const values = hrpExpand(hrp).concat(data).concat([0, 0, 0, 0, 0, 0]);
    const mod = polymod(values) ^ enc.const;
    const ret = [];
    for (let p = 0; p < 6; ++p) {
        ret.push((mod >> 5 * (5 - p)) & 31);
    }
    return ret;
}
function convertBits(data, fromBits, toBits, pad = true) {
    const ret = [];
    let acc = 0;
    let bits = 0;
    const maxVal = (1 << toBits) - 1;
    const maxAcc = (1 << (fromBits + toBits - 1)) - 1;
    for (const val of data) {
        if (val < 0 || (val >> fromBits) > 0) {
            throw new Error('Failed to perform base conversion. Invalid value: ' + String(val));
        }
        acc = ((acc << fromBits) | val) & maxAcc;
        bits += fromBits;
        while (bits >= toBits) {
            bits -= toBits;
            ret.push((acc >> bits) & maxVal);
        }
    }
    if (pad) {
        if (bits > 0) {
            ret.push((acc << (toBits - bits)) & maxVal);
        }
    }
    else if (bits >= fromBits || ((acc << (toBits - bits)) & maxVal) > 0) {
        throw new Error('Failed to perform base conversion. Invalid Size!');
    }
    return ret;
}
function encode(hrp, data, enc) {
    const combined = data.concat(createChecksum(hrp, data, enc));
    let ret = hrp + '1';
    for (let p = 0; p < combined.length; ++p) {
        ret += CHARSET.charAt(combined[p]);
    }
    return ret;
}
function decode(bechstr) {
    if (!checkBounds(bechstr)) {
        throw new Error('Encoded string goes out of bounds!');
    }
    bechstr = bechstr.toLowerCase();
    if (!checkSeparatorPos(bechstr)) {
        throw new Error('Encoded string has invalid separator!');
    }
    const data = [];
    const pos = bechstr.lastIndexOf('1');
    const hrp = bechstr.substring(0, pos);
    for (let p = pos + 1; p < bechstr.length; ++p) {
        const d = CHARSET.indexOf(bechstr.charAt(p));
        if (d === -1) {
            throw new Error('Character idx out of bounds: ' + String(p));
        }
        data.push(d);
    }
    const enc = ENCODINGS.find(e => e.version === data[0]) ?? ENCODINGS[0];
    if (!verifyChecksum(hrp, data, enc)) {
        throw new Error('Checksum verification failed!');
    }
    return [hrp, data.slice(0, data.length - 6)];
}
function checkBounds(bechstr) {
    let p;
    let char;
    let hasLower = false;
    let hasUpper = false;
    for (p = 0; p < bechstr.length; ++p) {
        char = bechstr.charCodeAt(p);
        if (char < 33 || char > 126) {
            return false;
        }
        if (char >= 97 && char <= 122) {
            hasLower = true;
        }
        if (char >= 65 && char <= 90) {
            hasUpper = true;
        }
    }
    if (hasLower && hasUpper)
        return false;
    return true;
}
function checkSeparatorPos(bechstr) {
    const pos = bechstr.lastIndexOf('1');
    return !(pos < 1 ||
        pos + 7 > bechstr.length ||
        bechstr.length > 90);
}
function b32encode(data, hrp = 'bc', version = 0) {
    const dat = [version, ...convertBits([...data], 8, 5)];
    const enc = ENCODINGS.find(e => e.version === version) ?? ENCODINGS[0];
    const str = encode(hrp, dat, enc);
    b32decode(str);
    return str;
}
function b32decode(str) {
    str = str.toLowerCase();
    const hrp = str.split('1', 1)[0];
    const [hrpgot, data] = decode(str);
    const decoded = convertBits(data.slice(1), 5, 8, false);
    const length = decoded.length;
    switch (true) {
        case (hrp !== hrpgot):
            throw new Error('Returned hrp string is invalid.');
        case (decoded === null || length < 2 || length > 40):
            throw new Error('Decoded string is invalid or out of spec.');
        case (data[0] > 16):
            throw new Error('Returned version bit is out of range.');
        default:
            return Uint8Array.from(decoded);
    }
}
function getVersion(str) {
    str = str.toLowerCase();
    const [_, data] = decode(str);
    return data[0];
}
const Bech32 = {
    encode: b32encode,
    decode: b32decode,
    version: getVersion
};

const BASE64_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
const B64URL_MAP = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_';
const ec$1 = new TextEncoder();
function b64encode(input, urlSafe = false) {
    if (typeof input === 'string')
        input = ec$1.encode(input);
    const map = urlSafe ? B64URL_MAP : BASE64_MAP;
    let output = '';
    let bits = 0;
    let buffer = 0;
    for (let i = 0; i < input.length; i++) {
        buffer = (buffer << 8) | input[i];
        bits += 8;
        while (bits >= 6) {
            bits -= 6;
            output += map[(buffer >> bits) & 0x3f];
        }
    }
    if (bits > 0) {
        buffer <<= 6 - bits;
        output += map[buffer & 0x3f];
        while (bits < 6) {
            output += urlSafe ? '' : '=';
            bits += 2;
        }
    }
    return output;
}
function b64decode(input, urlSafe = false) {
    const map = (urlSafe || input.includes('-') || input.includes('_'))
        ? B64URL_MAP.split('')
        : BASE64_MAP.split('');
    input = input.replace(/=+$/, '');
    const chars = input.split('');
    let bits = 0;
    let value = 0;
    const bytes = [];
    for (let i = 0; i < chars.length; i++) {
        const c = chars[i];
        const index = map.indexOf(c);
        if (index === -1) {
            throw new Error('Invalid character: ' + c);
        }
        bits += 6;
        value <<= 6;
        value |= index;
        if (bits >= 8) {
            bits -= 8;
            bytes.push((value >>> bits) & 0xff);
        }
    }
    return new Uint8Array(bytes);
}
const Base64 = {
    encode: b64encode,
    decode: b64decode
};
const B64URL = {
    encode: (data) => b64encode(data, true),
    decode: (data) => b64decode(data, true)
};

const webcrypto = getCryptoModule();
function getCryptoModule() {
    let webcrypto = null;
    if (exists(crypto.webcrypto)) {
        webcrypto = crypto.webcrypto;
    }
    else if (exists(globalThis.crypto)) {
        webcrypto = globalThis.crypto;
    }
    else if (exists(window.crypto)) {
        webcrypto = window.crypto;
    }
    if (!exists(webcrypto)) {
        console.log('Search for webcrypto library failed!');
        console.log('crypto:', typeof crypto.webcrypto);
        console.log('globalThis.crypto:', typeof globalThis?.crypto);
        console.log('window.crypto:', typeof window?.crypto);
        throw new Error('Unable to find webcrypto library. If you are running in a NodeJs environment, try using version 19 or newer. If you are running in a browser environment, make sure that you are running in a secure context.');
    }
    if (!exists(webcrypto?.subtle)) {
        throw new Error('Webcrypto library is partially disabled! Make sure you are running within a secure environment. If you are getting this error in the browser, make sure https is enabled and/or you are testing locally using 127.0.0.1.');
    }
    return webcrypto;
}
function exists(x) {
    try {
        return (typeof x !== 'undefined' && x !== null);
    }
    catch {
        return false;
    }
}

const ec = new TextEncoder();
const dc = new TextDecoder();
function strToBytes(str) {
    return ec.encode(str);
}
function hexToBytes(str) {
    const bytes = new Uint8Array(str.length / 2);
    let i, idx = 0;
    if (str.match(/[^a-fA-f0-9]/) !== null) {
        throw new TypeError('Invalid hex string: ' + str);
    }
    if (str.length % 2 > 0) {
        throw new Error(`Hex string length is uneven: ${str.length}`);
    }
    for (i = 0; i < str.length; i += 2) {
        bytes[idx] = parseInt(str.slice(i, i + 2), 16);
        idx += 1;
    }
    return bytes;
}
function numToBytes(num) {
    if (num === 0)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xff;
        bytes.push(byte);
        num = (num - byte) / 256;
    }
    return new Uint8Array(bytes);
}
function binaryToBytes(binary) {
    if (typeof binary === 'string') {
        binary = binary.split('').map(Number);
    }
    else if (!Array.isArray(binary)) {
        throw new Error('Invalid input type: expected a string or an array of numbers.');
    }
    if (binary.length % 8 !== 0) {
        throw new Error(`Binary array is invalid length: ${binary.length}`);
    }
    const bytes = new Uint8Array(binary.length / 8);
    for (let i = 0, ct = 0; i < binary.length; i += 8, ct++) {
        let byte = 0;
        for (let j = 0; j < 8; j++) {
            byte |= (binary[i + j] << (7 - j));
        }
        bytes[ct] = byte;
    }
    return bytes;
}
function bytesToBinary(bytes) {
    const bin = new Array(bytes.length * 8);
    let count = 0;
    for (const num of bytes) {
        if (num > 255) {
            throw new Error(`Invalid byte value: ${num}. Byte values must be between 0 and 255.`);
        }
        for (let i = 7; i >= 0; i--, count++) {
            bin[count] = (num >> i) & 1;
        }
    }
    return bin;
}
function bigToBytes(big) {
    if (big === 0n)
        return Uint8Array.of(0x00);
    const bytes = [];
    while (big > 0n) {
        const byte = big & 0xffn;
        bytes.push(Number(byte));
        big = (big - byte) / 256n;
    }
    return new Uint8Array(bytes);
}
function bytesToStr(bytes) {
    return dc.decode(bytes);
}
function bytesToHex(bytes) {
    const chars = new Array(bytes.length);
    for (let i = 0; i < bytes.length; i++) {
        chars.push(bytes[i].toString(16).padStart(2, '0'));
    }
    return chars.join('');
}
function bytesToNum(bytes) {
    let num = 0, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256) + bytes[i];
    }
    return Number(num);
}
function bytesToBig(bytes) {
    let num = 0n, i;
    for (i = bytes.length - 1; i >= 0; i--) {
        num = (num * 256n) + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function buffer(value, bytes = true) {
    if (value instanceof ArrayBuffer) {
        return new Uint8Array(value);
    }
    if (value instanceof Uint8Array) {
        return value;
    }
    switch (typeof value) {
        case 'bigint':
            return bigToBytes(value);
        case 'boolean':
            return Uint8Array.of(value ? 1 : 0);
        case 'number':
            return numToBytes(value);
        case 'string':
            return (bytes)
                ? hexToBytes(value)
                : ec.encode(value);
        default:
            throw TypeError('Unsupported format:' + String(typeof value));
    }
}
function hexify(bytes) {
    bytes = buffer(bytes, true);
    return bytesToHex(bytes);
}
function serialize(data) {
    if (typeof data === 'object') {
        if (data instanceof Uint8Array) {
            return data;
        }
        try {
            return strToBytes(JSON.stringify(data));
        }
        catch {
            throw TypeError('Object is not serializable.');
        }
    }
    return buffer(data, false);
}
function revive(data) {
    if (data instanceof Uint8Array) {
        data = bytesToStr(data);
    }
    if (typeof data === 'string') {
        try {
            return JSON.parse(data);
        }
        catch {
            return data;
        }
    }
    return data;
}

function bytesToBigInt$1(bytes) {
    let num = 0n;
    for (let i = bytes.length - 1; i >= 0; i--) {
        num = num * 256n + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function bigIntToBytes$1(num, size = 0) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xffn;
        bytes.push(byte);
        num = (num - byte) / 256n;
    }
    let uint8 = Uint8Array.from(bytes.map((n) => Number(n)));
    if (size !== 0) {
        const resized = new Uint8Array(size);
        resized.set(uint8);
        uint8 = resized;
    }
    return uint8;
}
const ML$1 = [
    0n,
    1n,
    2n,
    3n,
    4n,
    5n,
    6n,
    7n,
    8n,
    9n,
    10n,
    11n,
    12n,
    13n,
    14n,
    15n,
    7n,
    4n,
    13n,
    1n,
    10n,
    6n,
    15n,
    3n,
    12n,
    0n,
    9n,
    5n,
    2n,
    14n,
    11n,
    8n,
    3n,
    10n,
    14n,
    4n,
    9n,
    15n,
    8n,
    1n,
    2n,
    7n,
    0n,
    6n,
    13n,
    11n,
    5n,
    12n,
    1n,
    9n,
    11n,
    10n,
    0n,
    8n,
    12n,
    4n,
    13n,
    3n,
    7n,
    15n,
    14n,
    5n,
    6n,
    2n,
    4n,
    0n,
    5n,
    9n,
    7n,
    12n,
    2n,
    10n,
    14n,
    1n,
    3n,
    8n,
    11n,
    6n,
    15n,
    13n
];
const MR$1 = [
    5n,
    14n,
    7n,
    0n,
    9n,
    2n,
    11n,
    4n,
    13n,
    6n,
    15n,
    8n,
    1n,
    10n,
    3n,
    12n,
    6n,
    11n,
    3n,
    7n,
    0n,
    13n,
    5n,
    10n,
    14n,
    15n,
    8n,
    12n,
    4n,
    9n,
    1n,
    2n,
    15n,
    5n,
    1n,
    3n,
    7n,
    14n,
    6n,
    9n,
    11n,
    8n,
    12n,
    2n,
    10n,
    0n,
    4n,
    13n,
    8n,
    6n,
    4n,
    1n,
    3n,
    11n,
    15n,
    0n,
    5n,
    12n,
    2n,
    13n,
    9n,
    7n,
    10n,
    14n,
    12n,
    15n,
    10n,
    4n,
    1n,
    5n,
    8n,
    7n,
    6n,
    2n,
    13n,
    14n,
    0n,
    3n,
    9n,
    11n
];
const RL$1 = [
    11n,
    14n,
    15n,
    12n,
    5n,
    8n,
    7n,
    9n,
    11n,
    13n,
    14n,
    15n,
    6n,
    7n,
    9n,
    8n,
    7n,
    6n,
    8n,
    13n,
    11n,
    9n,
    7n,
    15n,
    7n,
    12n,
    15n,
    9n,
    11n,
    7n,
    13n,
    12n,
    11n,
    13n,
    6n,
    7n,
    14n,
    9n,
    13n,
    15n,
    14n,
    8n,
    13n,
    6n,
    5n,
    12n,
    7n,
    5n,
    11n,
    12n,
    14n,
    15n,
    14n,
    15n,
    9n,
    8n,
    9n,
    14n,
    5n,
    6n,
    8n,
    6n,
    5n,
    12n,
    9n,
    15n,
    5n,
    11n,
    6n,
    8n,
    13n,
    12n,
    5n,
    12n,
    13n,
    14n,
    11n,
    8n,
    5n,
    6n
];
const RR$1 = [
    8n,
    9n,
    9n,
    11n,
    13n,
    15n,
    15n,
    5n,
    7n,
    7n,
    8n,
    11n,
    14n,
    14n,
    12n,
    6n,
    9n,
    13n,
    15n,
    7n,
    12n,
    8n,
    9n,
    11n,
    7n,
    7n,
    12n,
    7n,
    6n,
    15n,
    13n,
    11n,
    9n,
    7n,
    15n,
    11n,
    8n,
    6n,
    6n,
    14n,
    12n,
    13n,
    5n,
    14n,
    13n,
    13n,
    7n,
    5n,
    15n,
    5n,
    8n,
    11n,
    14n,
    14n,
    6n,
    14n,
    6n,
    9n,
    12n,
    9n,
    12n,
    5n,
    15n,
    8n,
    8n,
    5n,
    12n,
    9n,
    12n,
    5n,
    14n,
    6n,
    8n,
    13n,
    6n,
    5n,
    15n,
    13n,
    11n,
    11n
];
const KL$1 = [0n, 0x5a827999n, 0x6ed9eba1n, 0x8f1bbcdcn, 0xa953fd4en];
const KR$1 = [0x50a28be6n, 0x5c4dd124n, 0x6d703ef3n, 0x7a6d76e9n, 0n];
function fi$1(x, y, z, i) {
    switch (true) {
        case i === 0n:
            return x ^ y ^ z;
        case i === 1n:
            return (x & y) | (~x & z);
        case i === 2n:
            return (x | ~y) ^ z;
        case i === 3n:
            return (x & z) | (y & ~z);
        case i === 4n:
            return x ^ (y | ~z);
        default:
            throw new TypeError('Unknown I value: ' + String(i));
    }
}
function rol$1(x, i) {
    return ((x << i) | ((x & 0xffffffffn) >> (32n - i))) & 0xffffffffn;
}
function compress$1(h0, h1, h2, h3, h4, block) {
    const x = [];
    let rnd, elt, ert;
    let ml, kl, mr, kr;
    let al = h0, bl = h1, cl = h2, dl = h3, el = h4;
    let ar = h0, br = h1, cr = h2, dr = h3, er = h4;
    for (let i = 0; i < 16; i++) {
        const num = bytesToBigInt$1(block.slice(4 * i, 4 * (i + 1)));
        x.push(num);
    }
    for (let i = 0; i < 80; i++) {
        rnd = BigInt(i) >> 4n;
        ml = x[Number(ML$1[i])];
        kl = KL$1[Number(rnd)];
        mr = x[Number(MR$1[i])];
        kr = KR$1[Number(rnd)];
        al = rol$1(al + fi$1(bl, cl, dl, rnd) + ml + kl, RL$1[i]) + el;
        elt = el;
        el = dl;
        dl = rol$1(cl, 10n);
        cl = bl;
        bl = al;
        al = elt;
        ar = rol$1(ar + fi$1(br, cr, dr, 4n - rnd) + mr + kr, RR$1[i]) + er;
        ert = er;
        er = dr;
        dr = rol$1(cr, 10n);
        cr = br;
        br = ar;
        ar = ert;
    }
    return [h1 + cl + dr, h2 + dl + er, h3 + el + ar, h4 + al + br, h0 + bl + cr];
}
function ripemd160$1(bytes) {
    let state = [
        0x67452301n,
        0xefcdab89n,
        0x98badcfen,
        0x10325476n,
        0xc3d2e1f0n
    ];
    for (let b = 0; b < bytes.length >> 6; b++) {
        state = compress$1(...state, bytes.slice(64 * b, 64 * (b + 1)));
    }
    const zfill = new Array((119 - bytes.length) & 63).fill(0);
    const pad = [0x80, ...zfill];
    const fin = Uint8Array.from([
        ...bytes.slice(bytes.length & ~63),
        ...pad,
        ...bigIntToBytes$1(BigInt(8 * bytes.length), 8)
    ]);
    for (let i = 0; i < fin.length >> 6; i++) {
        state = compress$1(...state, fin.slice(64 * i, 64 * (i + 1)));
    }
    const ret = [];
    for (let i = 0; i < state.length; i++) {
        const num = state[i] & 0xffffffffn;
        ret.push(...bigIntToBytes$1(num, 4));
    }
    return Uint8Array.from(ret);
}

class Buff extends Uint8Array {
    static { this.num = (number, size, endian = 'le') => {
        const b = new Buff(numToBytes(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.big = (number, size, endian = 'le') => {
        const b = new Buff(bigToBytes(number), size);
        return (endian === 'le') ? b.reverse() : b;
    }; }
    static { this.bin = (data, size) => new Buff(binaryToBytes(data), size); }
    static { this.any = (data, size) => new Buff(buffer(data, false), size); }
    static { this.raw = (data, size) => new Buff(data, size); }
    static { this.str = (data, size) => new Buff(strToBytes(data), size); }
    static { this.hex = (data, size) => new Buff(hexToBytes(data), size); }
    static { this.json = (data) => new Buff(strToBytes(JSON.stringify(data))); }
    static { this.bytes = (data, size) => new Buff(buffer(data, true), size); }
    static { this.base64 = (data) => new Buff(Base64.decode(data)); }
    static { this.b64url = (data) => new Buff(B64URL.decode(data)); }
    static { this.bech32 = (data) => new Buff(Bech32.decode(data)); }
    static { this.b58chk = (data) => new Buff(Base58C.decode(data)); }
    constructor(data, size) {
        data = buffer(data, true);
        if (typeof size === 'number') {
            const tmp = new Uint8Array(size).fill(0);
            tmp.set(new Uint8Array(data));
            data = tmp.buffer;
        }
        super(data);
        return this;
    }
    get arr() {
        return [...this];
    }
    get num() {
        return this.toNum();
    }
    get big() {
        return this.toBig();
    }
    get str() {
        return this.toStr();
    }
    get hex() {
        return this.toHex();
    }
    get raw() {
        return new Uint8Array(this);
    }
    get bits() {
        return this.toBits();
    }
    get bin() {
        return this.toBin();
    }
    get b58chk() {
        return this.tob58chk();
    }
    get base64() {
        return this.toBase64();
    }
    get b64url() {
        return this.toB64url();
    }
    get digest() {
        return this.toHash();
    }
    get id() {
        return this.toHash().hex;
    }
    get stream() {
        return new Stream(this);
    }
    toNum(endian = 'le') {
        return (endian === 'le')
            ? bytesToNum(this.reverse())
            : bytesToNum(this);
    }
    toBig(endian = 'le') {
        return (endian === 'le')
            ? bytesToBig(this.reverse())
            : bytesToBig(this);
    }
    toHash(type = 'sha256') {
        switch (type) {
            case 'sha256':
                return new Buff(sha256$1(this));
            case 'hash256':
                return new Buff(sha256$1(sha256$1(this)));
            case 'ripe160':
                return new Buff(ripemd160$1(this));
            case 'hash160':
                return new Buff(ripemd160$1(sha256$1(this)));
            default:
                throw new Error('Unrecognized format:' + String(type));
        }
    }
    toHmac(key) {
        if (typeof key === 'string') {
            key = strToBytes(key);
        }
        return new Buff(hmac256$1(key, this));
    }
    toStr() { return bytesToStr(this); }
    toHex() { return bytesToHex(this); }
    toJson() { return JSON.parse(bytesToStr(this)); }
    toBytes() { return new Uint8Array(this); }
    toBits() { return bytesToBinary(this); }
    toBin() { return bytesToBinary(this).join(''); }
    tob58chk() { return Base58C.encode(this); }
    toB64url() { return B64URL.encode(this); }
    toBase64() { return Base64.encode(this); }
    toBech32(hrp, version = 0) { return Bech32.encode(this, hrp, version); }
    prepend(data) {
        return Buff.join([Buff.bytes(data), this]);
    }
    append(data) {
        return Buff.join([this, Buff.bytes(data)]);
    }
    slice(start, end) {
        return new Buff(new Uint8Array(this).slice(start, end));
    }
    reverse() {
        return new Buff(new Uint8Array(this).reverse());
    }
    write(bytes, offset) {
        this.set(bytes, offset);
    }
    prefixSize(endian) {
        const size = Buff.varInt(this.length, endian);
        return Buff.join([size, this]);
    }
    static from(data) {
        return new Buff(Uint8Array.from(data));
    }
    static of(...args) {
        return new Buff(Uint8Array.of(...args));
    }
    static join(arr) {
        const data = arr.map(e => Buff.bytes(e));
        return new Buff(joinArray(data));
    }
    static varInt(num, endian) {
        if (num < 0xFD) {
            return Buff.num(num, 1);
        }
        else if (num < 0x10000) {
            return Buff.of(0xFD, ...Buff.num(num, 2, endian));
        }
        else if (num < 0x100000000) {
            return Buff.of(0xFE, ...Buff.num(num, 4, endian));
        }
        else if (BigInt(num) < 0x10000000000000000n) {
            return Buff.of(0xFF, ...Buff.num(num, 8, endian));
        }
        else {
            throw new Error(`Value is too large: ${num}`);
        }
    }
    static { this.encode = strToBytes; }
    static { this.decode = bytesToStr; }
    static random(size = 32) {
        return new Buff(webcrypto.getRandomValues(new Uint8Array(size)));
    }
    static normalize(bytes, size) {
        return new Buff(buffer(bytes, true), size);
    }
    static hexify(bytes) {
        return hexify(bytes);
    }
    static serialize(data, size) {
        return new Buff(serialize(data), size);
    }
    static revive(data) {
        return revive(data);
    }
}
class Stream {
    constructor(data) {
        this.data = new Uint8Array(data);
        this.size = this.data.length;
    }
    peek(size) {
        if (size > this.size) {
            throw new Error(`Size greater than stream: ${size} > ${this.size}`);
        }
        return new Buff(this.data.slice(0, size).buffer);
    }
    read(size) {
        size = size ?? this.readSize();
        const chunk = this.peek(size);
        this.data = this.data.slice(size);
        this.size = this.data.length;
        return chunk;
    }
    readSize(endian) {
        const num = this.read(1).num;
        switch (true) {
            case (num >= 0 && num < 0xFD):
                return num;
            case (num === 0xFD):
                return this.read(2).toNum(endian);
            case (num === 0xFE):
                return this.read(4).toNum(endian);
            case (num === 0xFF):
                return this.read(8).toNum(endian);
            default:
                throw new Error(`Varint is out of range: ${num}`);
        }
    }
}

async function ecdh(secretKey, sharedKey) {
    const secret = Buff.normalize(secretKey);
    const shared = Buff.normalize(sharedKey);
    return getSharedSecret(secret, shared, true);
}
async function importCryptoKey(key) {
    const options = { name: 'AES-CBC' };
    const usage = ['encrypt', 'decrypt'];
    return webcrypto$1.subtle.importKey('raw', Buff.normalize(key), options, true, usage);
}
async function exportCryptoKey(key) {
    return webcrypto$1.subtle.exportKey('raw', key)
        .then(buff => new Uint8Array(buff));
}
async function importHmacKey(key, fmt = 'SHA-256') {
    const config = { name: 'HMAC', hash: fmt };
    return webcrypto$1.subtle.importKey('raw', Buff.normalize(key), config, false, ['sign', 'verify']);
}
async function generateCryptoKey() {
    return importCryptoKey(Buff.random(32));
}
async function normalizeCryptoKey(key) {
    if (key instanceof CryptoKey)
        return key;
    return KeyUtil.import(key);
}
async function getSharedCryptoKey(secretKey, sharedKey) {
    const bytes = await ecdh(secretKey, sharedKey);
    return importCryptoKey(bytes.slice(1, 33));
}
function getXOnlyPub(bytes) {
    const b = Buff.bytes(bytes);
    if (b.length === 33) {
        return b.slice(1, 33);
    }
    if (b.length === 32) {
        return b;
    }
    throw new Error('Invalid key length: ' + String(b.length));
}
const KeyUtil = {
    ecdh,
    import: importCryptoKey,
    export: exportCryptoKey,
    hmac: importHmacKey,
    shared: getSharedCryptoKey,
    generate: generateCryptoKey,
    normalize: normalizeCryptoKey
};

class Cipher {
    static async fromSecret(secret) {
        const cryptoKey = await KeyUtil.normalize(secret);
        return new Cipher(cryptoKey);
    }
    static async fromShared(secret, shared) {
        const cryptoKey = await KeyUtil.shared(secret, shared);
        return new Cipher(cryptoKey);
    }
    static async encrypt(secret, message, vector) {
        const msg = Buff.normalize(message);
        const key = await KeyUtil.normalize(secret);
        const iv = (vector !== undefined) ? Buff.normalize(vector) : Buff.random(16);
        return webcrypto$1.subtle
            .encrypt({ name: 'AES-CBC', iv }, key, msg)
            .then((buffer) => Uint8Array.of(...iv, ...new Uint8Array(buffer)));
    }
    static async decrypt(secret, message, vector) {
        const msg = Buff.normalize(message);
        const key = await KeyUtil.normalize(secret);
        const dat = (vector !== undefined) ? msg : msg.slice(16);
        const iv = (vector !== undefined) ? Buff.normalize(vector) : msg.slice(0, 16);
        return webcrypto$1.subtle
            .decrypt({ name: 'AES-CBC', iv }, key, dat)
            .then((buffer) => new Uint8Array(buffer));
    }
    constructor(cryptoKey) {
        this.key = cryptoKey;
    }
    get secretKey() {
        return KeyUtil.export(this.key);
    }
    get secretHex() {
        return this.secretKey
            .then(b => Buff.raw(b).hex);
    }
    async encrypt(message, vector) {
        return Cipher.encrypt(this.key, message, vector);
    }
    async decrypt(message, vector) {
        return Cipher.decrypt(this.key, message, vector);
    }
}

class Field extends Uint8Array {
    static { this.N = CURVE.n; }
    static mod(x, n = Field.N) {
        return utils.mod(x, n);
    }
    static pow(x, e, n = Field.N) {
        x = Field.mod(x, n);
        e = Field.mod(e, n);
        if (x === 0n)
            return 0n;
        let res = 1n;
        while (e > 0n) {
            if ((e & 1n) === 1n) {
                res = Field.mod(res * x, n);
            }
            e = e >> 1n;
            x = Field.mod(x * x, n);
        }
        return res;
    }
    static normalize(num) {
        num = normalizeField(num);
        num = Field.mod(num);
        Field.validate(num);
        return Buff.big(num, 32);
    }
    static validate(num) {
        return utils.isValidPrivateKey(num);
    }
    constructor(x) {
        super(Field.normalize(x), 32);
    }
    get buff() {
        return new Buff(this);
    }
    get raw() {
        return this.buff.raw;
    }
    get big() {
        return this.buff.big;
    }
    get hex() {
        return this.buff.hex;
    }
    get point() {
        return this.generate();
    }
    get xpoint() {
        return new Point(this.point.x);
    }
    get hasOddY() {
        return this.point.hasOddY;
    }
    get negated() {
        return (this.hasOddY)
            ? this.negate()
            : this;
    }
    gt(big) {
        const x = new Field(big);
        return x.big > this.big;
    }
    lt(big) {
        const x = new Field(big);
        return x.big < this.big;
    }
    eq(big) {
        const x = new Field(big);
        return x.big === this.big;
    }
    ne(big) {
        const x = new Field(big);
        return x.big !== this.big;
    }
    add(big) {
        const x = new Field(big);
        return new Field(this.big + x.big);
    }
    sub(big) {
        const x = new Field(big);
        return new Field(this.big - x.big);
    }
    mul(big) {
        const x = new Field(big);
        return new Field(this.big * x.big);
    }
    pow(big, n = Field.N - 1n) {
        const x = new Field(big);
        const e = Field.mod(x.big, n);
        return new Field(this.big ** e);
    }
    div(big) {
        const x = new Field(big);
        const d = this.pow(x.big, Field.N - 2n);
        return new Field(this.big * d.big);
    }
    negate() {
        return new Field(Field.N - this.big);
    }
    generate() {
        return Point.import(Point$1.BASE.multiply(this.big));
    }
}
class Point {
    static { this.N = CURVE.n; }
    static validate(x) {
        try {
            x = new Point(x);
            return true;
        }
        catch {
            return false;
        }
    }
    static normalize(x) {
        x = normalizePoint(x);
        return Point$1.fromHex(x);
    }
    static generate(big) {
        return new Field(big).generate();
    }
    static import(point) {
        const p = (point instanceof Point)
            ? { x: point.x.big, y: point.y.big }
            : point;
        return new Point(p.x, p.y);
    }
    constructor(x, y) {
        this.__p = (typeof x === 'bigint' &&
            typeof y === 'bigint')
            ? new Point$1(x, y)
            : Point.normalize(x);
        this.p.assertValidity();
    }
    get p() {
        return this.__p;
    }
    get x() {
        return Buff.big(this.p.x, 32);
    }
    get y() {
        return Buff.big(this.p.y, 32);
    }
    get buff() {
        const p = this.p.toRawBytes(true);
        if (p.length < 33) {
            const prefix = this.p.hasEvenY() ? 0x02 : 0x03;
            return Buff.join([prefix, p]);
        }
        else {
            return Buff.raw(p);
        }
    }
    get raw() {
        return this.buff.raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get hasEvenY() {
        return this.p.hasEvenY();
    }
    get hasOddY() {
        return !this.p.hasEvenY();
    }
    eq(value) {
        return (value instanceof Point)
            ? this.p.equals(new Point$1(value.x.big, value.y.big))
            : (value instanceof Uint8Array)
                ? this.x.big === Buff.raw(value).big
                : (typeof value === 'number')
                    ? BigInt(value) === this.x.big
                    : value === this.x.big;
    }
    add(x) {
        return (x instanceof Point)
            ? Point.import(this.p.add(x.p))
            : Point.import(this.p.add(Point.generate(x).p));
    }
    sub(x) {
        return (x instanceof Point)
            ? Point.import(this.p.subtract(x.p))
            : Point.import(this.p.subtract(Point.generate(x).p));
    }
    mul(value) {
        return (value instanceof Point)
            ? Point.import(this.p.multiply(value.x.big))
            : Point.import(this.p.multiply(normalizeField(value)));
    }
    negate() {
        return Point.import(this.p.negate());
    }
}
function normalizeField(value) {
    if (value instanceof Field) {
        return value.big;
    }
    if (value instanceof Point) {
        return value.x.big;
    }
    if (value instanceof Uint8Array) {
        return Buff.raw(value).big;
    }
    if (typeof value === 'string') {
        return Buff.hex(value).big;
    }
    if (typeof value === 'number') {
        return Buff.num(value).big;
    }
    if (typeof value === 'bigint') {
        return BigInt(value);
    }
    throw TypeError('Invalid input type:' + typeof value);
}
function normalizePoint(value) {
    if (value instanceof Field) {
        return value.hex;
    }
    if (value instanceof Point) {
        return value.hex;
    }
    if (value instanceof Uint8Array) {
        return Buff.raw(value).hex;
    }
    if (typeof value === 'string') {
        return value;
    }
    if (typeof value === 'number') {
        return Buff.num(value).hex;
    }
    if (typeof value === 'bigint') {
        return Buff.big(value).hex;
    }
    throw TypeError('Invalid input type:' + typeof value);
}

async function sign(secret, message, type = 'taproot') {
    const msg = Buff.bytes(message).raw;
    const key = Buff.bytes(secret).raw;
    switch (type) {
        case 'ecdsa':
            return sign$1(msg, key);
        case 'taproot':
            return schnorr.sign(msg, key);
        default:
            throw new Error('Unknown signature type:' + String(type));
    }
}
async function verify(signature, message, pubkey, type = 'taproot') {
    const sig = Buff.bytes(signature).raw;
    const msg = Buff.bytes(message).raw;
    const pub = Buff.bytes(pubkey).raw;
    switch (type) {
        case 'ecdsa':
            return verify$1(sig, msg, pub);
        case 'taproot':
            return schnorr.verify(sig, msg, getXOnlyPub(pub));
        default:
            throw new Error('Unknown signature type:' + String(type));
    }
}

const DEFAULT_CONFIG = {
    type: 'ecdsa'
};
class SecretKey extends Uint8Array {
    static random(opt) {
        return new SecretKey(Buff.random(32), opt);
    }
    constructor(secret, options = {}) {
        super(new Field(secret));
        this.config = { ...DEFAULT_CONFIG, ...options };
        this.xonly = this.config.type === 'taproot';
    }
    get buff() {
        return new Buff(this);
    }
    get raw() {
        return this.buff.raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get field() {
        return new Field(this);
    }
    get point() {
        return this.field.point;
    }
    get pub() {
        return new PublicKey(this.point.raw, this.config);
    }
    get hasEvenY() {
        return this.point.hasEvenY;
    }
    get hasOddY() {
        return this.point.hasOddY;
    }
    get xfilter() {
        return (this.xonly && this.hasOddY)
            ? this.negate()
            : this;
    }
    add(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.add(bytes), this.config);
    }
    sub(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.sub(bytes), this.config);
    }
    mul(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.mul(bytes), this.config);
    }
    div(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.div(bytes), this.config);
    }
    pow(bytes) {
        const field = this.xfilter.field;
        return new SecretKey(field.pow(bytes), this.config);
    }
    negate() {
        return new SecretKey(this.field.negate(), this.config);
    }
    async sign(message, type = this.config.type) {
        return sign(this.raw, message, type);
    }
    async verify(signature, message, type = this.config.type) {
        return verify(signature, message, this.pub.raw, type);
    }
    toWIF(prefix = 0x80) {
        return Buff.join([prefix, this, 0x01]).b58chk;
    }
}
class PublicKey extends Uint8Array {
    static random(opt) {
        return SecretKey.random(opt).pub;
    }
    static fromSecret(bytes, opt) {
        return new SecretKey(bytes, opt).pub;
    }
    static { this.xfilter = getXOnlyPub; }
    constructor(pubkey, options = {}) {
        const config = { ...DEFAULT_CONFIG, ...options };
        if (config.type === 'taproot') {
            super(PublicKey.xfilter(pubkey), 32);
        }
        else {
            super(Buff.bytes(pubkey), 33);
        }
        this.config = { ...DEFAULT_CONFIG, ...options };
        this.xonly = config.type === 'taproot';
    }
    get buff() {
        return (this.xonly) ? this.x : new Buff(this);
    }
    get raw() {
        return this.buff.raw;
    }
    get hex() {
        return this.buff.hex;
    }
    get point() {
        return new Point(this);
    }
    get x() {
        return this.point.x;
    }
    get y() {
        return this.point.y;
    }
    get hasEvenY() {
        return this.point.hasEvenY;
    }
    get hasOddY() {
        return this.point.hasOddY;
    }
    add(bytes) {
        return new PublicKey(this.point.add(bytes).raw, this.config);
    }
    sub(bytes) {
        return new PublicKey(this.point.sub(bytes).raw, this.config);
    }
    mul(bytes) {
        return new PublicKey(this.point.mul(bytes).raw, this.config);
    }
    negate() {
        return new PublicKey(this.point.negate().raw, this.config);
    }
    async verify(signature, message, type = this.config.type) {
        return verify(signature, message, this.raw, type);
    }
}
const KeyPair = SecretKey;

function bytesToBigInt(bytes) {
    let num = 0n;
    for (let i = bytes.length - 1; i >= 0; i--) {
        num = num * 256n + BigInt(bytes[i]);
    }
    return BigInt(num);
}
function bigIntToBytes(num, size = 0) {
    const bytes = [];
    while (num > 0) {
        const byte = num & 0xffn;
        bytes.push(byte);
        num = (num - byte) / 256n;
    }
    let uint8 = Uint8Array.from(bytes.map((n) => Number(n)));
    if (size !== 0) {
        const resized = new Uint8Array(size);
        resized.set(uint8);
        uint8 = resized;
    }
    return uint8;
}
const ML = [
    0n,
    1n,
    2n,
    3n,
    4n,
    5n,
    6n,
    7n,
    8n,
    9n,
    10n,
    11n,
    12n,
    13n,
    14n,
    15n,
    7n,
    4n,
    13n,
    1n,
    10n,
    6n,
    15n,
    3n,
    12n,
    0n,
    9n,
    5n,
    2n,
    14n,
    11n,
    8n,
    3n,
    10n,
    14n,
    4n,
    9n,
    15n,
    8n,
    1n,
    2n,
    7n,
    0n,
    6n,
    13n,
    11n,
    5n,
    12n,
    1n,
    9n,
    11n,
    10n,
    0n,
    8n,
    12n,
    4n,
    13n,
    3n,
    7n,
    15n,
    14n,
    5n,
    6n,
    2n,
    4n,
    0n,
    5n,
    9n,
    7n,
    12n,
    2n,
    10n,
    14n,
    1n,
    3n,
    8n,
    11n,
    6n,
    15n,
    13n
];
const MR = [
    5n,
    14n,
    7n,
    0n,
    9n,
    2n,
    11n,
    4n,
    13n,
    6n,
    15n,
    8n,
    1n,
    10n,
    3n,
    12n,
    6n,
    11n,
    3n,
    7n,
    0n,
    13n,
    5n,
    10n,
    14n,
    15n,
    8n,
    12n,
    4n,
    9n,
    1n,
    2n,
    15n,
    5n,
    1n,
    3n,
    7n,
    14n,
    6n,
    9n,
    11n,
    8n,
    12n,
    2n,
    10n,
    0n,
    4n,
    13n,
    8n,
    6n,
    4n,
    1n,
    3n,
    11n,
    15n,
    0n,
    5n,
    12n,
    2n,
    13n,
    9n,
    7n,
    10n,
    14n,
    12n,
    15n,
    10n,
    4n,
    1n,
    5n,
    8n,
    7n,
    6n,
    2n,
    13n,
    14n,
    0n,
    3n,
    9n,
    11n
];
const RL = [
    11n,
    14n,
    15n,
    12n,
    5n,
    8n,
    7n,
    9n,
    11n,
    13n,
    14n,
    15n,
    6n,
    7n,
    9n,
    8n,
    7n,
    6n,
    8n,
    13n,
    11n,
    9n,
    7n,
    15n,
    7n,
    12n,
    15n,
    9n,
    11n,
    7n,
    13n,
    12n,
    11n,
    13n,
    6n,
    7n,
    14n,
    9n,
    13n,
    15n,
    14n,
    8n,
    13n,
    6n,
    5n,
    12n,
    7n,
    5n,
    11n,
    12n,
    14n,
    15n,
    14n,
    15n,
    9n,
    8n,
    9n,
    14n,
    5n,
    6n,
    8n,
    6n,
    5n,
    12n,
    9n,
    15n,
    5n,
    11n,
    6n,
    8n,
    13n,
    12n,
    5n,
    12n,
    13n,
    14n,
    11n,
    8n,
    5n,
    6n
];
const RR = [
    8n,
    9n,
    9n,
    11n,
    13n,
    15n,
    15n,
    5n,
    7n,
    7n,
    8n,
    11n,
    14n,
    14n,
    12n,
    6n,
    9n,
    13n,
    15n,
    7n,
    12n,
    8n,
    9n,
    11n,
    7n,
    7n,
    12n,
    7n,
    6n,
    15n,
    13n,
    11n,
    9n,
    7n,
    15n,
    11n,
    8n,
    6n,
    6n,
    14n,
    12n,
    13n,
    5n,
    14n,
    13n,
    13n,
    7n,
    5n,
    15n,
    5n,
    8n,
    11n,
    14n,
    14n,
    6n,
    14n,
    6n,
    9n,
    12n,
    9n,
    12n,
    5n,
    15n,
    8n,
    8n,
    5n,
    12n,
    9n,
    12n,
    5n,
    14n,
    6n,
    8n,
    13n,
    6n,
    5n,
    15n,
    13n,
    11n,
    11n
];
const KL = [0n, 0x5a827999n, 0x6ed9eba1n, 0x8f1bbcdcn, 0xa953fd4en];
const KR = [0x50a28be6n, 0x5c4dd124n, 0x6d703ef3n, 0x7a6d76e9n, 0n];
function fi(x, y, z, i) {
    switch (true) {
        case i === 0n:
            return x ^ y ^ z;
        case i === 1n:
            return (x & y) | (~x & z);
        case i === 2n:
            return (x | ~y) ^ z;
        case i === 3n:
            return (x & z) | (y & ~z);
        case i === 4n:
            return x ^ (y | ~z);
        default:
            throw new TypeError('Unknown I value: ' + String(i));
    }
}
function rol(x, i) {
    return ((x << i) | ((x & 0xffffffffn) >> (32n - i))) & 0xffffffffn;
}
function compress(h0, h1, h2, h3, h4, block) {
    const x = [];
    let rnd, elt, ert;
    let ml, kl, mr, kr;
    let al = h0, bl = h1, cl = h2, dl = h3, el = h4;
    let ar = h0, br = h1, cr = h2, dr = h3, er = h4;
    for (let i = 0; i < 16; i++) {
        const num = bytesToBigInt(block.slice(4 * i, 4 * (i + 1)));
        x.push(num);
    }
    for (let i = 0; i < 80; i++) {
        rnd = BigInt(i) >> 4n;
        ml = x[Number(ML[i])];
        kl = KL[Number(rnd)];
        mr = x[Number(MR[i])];
        kr = KR[Number(rnd)];
        al = rol(al + fi(bl, cl, dl, rnd) + ml + kl, RL[i]) + el;
        elt = el;
        el = dl;
        dl = rol(cl, 10n);
        cl = bl;
        bl = al;
        al = elt;
        ar = rol(ar + fi(br, cr, dr, 4n - rnd) + mr + kr, RR[i]) + er;
        ert = er;
        er = dr;
        dr = rol(cr, 10n);
        cr = br;
        br = ar;
        ar = ert;
    }
    return [h1 + cl + dr, h2 + dl + er, h3 + el + ar, h4 + al + br, h0 + bl + cr];
}
function ripemd160(bytes) {
    let state = [
        0x67452301n,
        0xefcdab89n,
        0x98badcfen,
        0x10325476n,
        0xc3d2e1f0n
    ];
    for (let b = 0; b < bytes.length >> 6; b++) {
        state = compress(...state, bytes.slice(64 * b, 64 * (b + 1)));
    }
    const zfill = new Array((119 - bytes.length) & 63).fill(0);
    const pad = [0x80, ...zfill];
    const fin = Uint8Array.from([
        ...bytes.slice(bytes.length & ~63),
        ...pad,
        ...bigIntToBytes(BigInt(8 * bytes.length), 8)
    ]);
    for (let i = 0; i < fin.length >> 6; i++) {
        state = compress(...state, fin.slice(64 * i, 64 * (i + 1)));
    }
    const ret = [];
    for (let i = 0; i < state.length; i++) {
        const num = state[i] & 0xffffffffn;
        ret.push(...bigIntToBytes(num, 4));
    }
    return Uint8Array.from(ret);
}

async function digest(bytes, format = 'SHA-256', rounds = 1, hook = (buff) => buff) {
    let i;
    let buffer = (bytes instanceof ArrayBuffer)
        ? bytes
        : Buff.normalize(bytes).buffer;
    for (i = 0; i < rounds; i++) {
        buffer = await webcrypto$1.subtle.digest(format, buffer);
        hook(buffer);
    }
    return new Uint8Array(buffer);
}
async function sha256(bytes) {
    return digest(bytes, 'SHA-256');
}
async function sha512(bytes) {
    return digest(bytes, 'SHA-512');
}
function ripe160(bytes) {
    bytes = Buff.normalize(bytes);
    return ripemd160(bytes);
}
async function hash256(bytes) {
    return digest(bytes, 'SHA-256', 2);
}
async function hash160(bytes) {
    return ripe160(await sha256(bytes));
}
async function hmac256(key, data) {
    const keyfile = await KeyUtil.hmac(key, 'SHA-256');
    return webcrypto$1.subtle
        .sign('HMAC', keyfile, Buff.normalize(data))
        .then((buffer) => new Uint8Array(buffer));
}
async function hmac512(key, data) {
    const keyfile = await KeyUtil.hmac(key, 'SHA-512');
    return webcrypto$1.subtle
        .sign('HMAC', keyfile, Buff.normalize(data))
        .then((buffer) => new Uint8Array(buffer));
}
async function data(data) {
    const buff = Buff.serialize(data);
    return sha256(buff).then(buff => new Uint8Array(buff));
}
const Hash = {
    data,
    digest,
    ripe160,
    sha256,
    sha512,
    hash160,
    hash256,
    hmac256,
    hmac512,
    sync: {
        sha256: sha256$1,
        hash256: hash256$1,
        hmac256: hmac256$1
    }
};

exports.Cipher = Cipher;
exports.Field = Field;
exports.Hash = Hash;
exports.KeyPair = KeyPair;
exports.KeyUtil = KeyUtil;
exports.Noble = module$1;
exports.Point = Point;
exports.PublicKey = PublicKey;
exports.SecretKey = SecretKey;
exports.getXOnlyPub = getXOnlyPub;
exports.sign = sign;
exports.verify = verify;
//# sourceMappingURL=main.js.map
